@@ -1,6 +1,6 @@
 /* decavcodec.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2020 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -107,6 +107,7 @@ struct video_filters_s
 
     int                   width;
     int                   height;
+    int                   color_range;
     int                   pix_fmt;
 };
 
@@ -118,6 +119,7 @@ struct hb_work_private_s
     AVCodecContext       * context;
     AVCodecParserContext * parser;
     AVFrame              * frame;
+    AVPacket             * pkt;
     hb_buffer_t          * palette;
     int                    threads;
     int                    video_codec_opened;
@@ -209,7 +211,7 @@ static int decavcodecaInit( hb_work_object_t * w, hb_job_t * job )
         // Currently, samplerate conversion is performed in sync.c
         // So set output samplerate to input samplerate
         // This should someday get reworked to be part of an audio
-        // filter pipleine.
+        // filter pipeline.
         pv->resample =
             hb_audio_resample_init(AV_SAMPLE_FMT_FLT,
                                    w->audio->config.in.samplerate,
@@ -336,6 +338,13 @@ static int decavcodecaInit( hb_work_object_t * w, hb_job_t * job )
         return 1;
     }
 
+    pv->pkt = av_packet_alloc();
+    if (pv->pkt == NULL)
+    {
+        hb_log("decavcodecaInit: av_packet_alloc failed");
+        return 1;
+    }
+
     return 0;
 }
 
@@ -395,6 +404,7 @@ static void closePrivData( hb_work_private_t ** ppv )
         {
             hb_avcodec_free_context(&pv->context);
         }
+        av_packet_free(&pv->pkt);
         hb_audio_resample_free(pv->resample);
 
         int ii;
@@ -735,16 +745,15 @@ static int decavcodecaBSInfo( hb_work_object_t *w, const hb_buffer_t *buf,
                 context->request_channel_layout = 0;
             }
 
-            AVPacket avp;
-            av_init_packet(&avp);
-            avp.data = parse_buffer;
-            avp.size = parse_buffer_size;
+            AVPacket *avp = av_packet_alloc();
+            avp->data = parse_buffer;
+            avp->size = parse_buffer_size;
 
-            ret = avcodec_send_packet(context, &avp);
+            ret = avcodec_send_packet(context, avp);
             if (ret < 0 && ret != AVERROR_EOF)
             {
                 parse_pos += parse_len;
-                av_packet_unref(&avp);
+                av_packet_free(&avp);
                 continue;
             }
 
@@ -850,7 +859,7 @@ static int decavcodecaBSInfo( hb_work_object_t *w, const hb_buffer_t *buf,
                         // Parse ADTS AAC streams for AudioSpecificConfig.
                         // This data is required in order to write
                         // proper headers in MP4, WebM, and MKV files.
-                        parse_adts_extradata(audio, context, &avp);
+                        parse_adts_extradata(audio, context, avp);
                     }
 
                     result = 1;
@@ -859,7 +868,7 @@ static int decavcodecaBSInfo( hb_work_object_t *w, const hb_buffer_t *buf,
                     break;
                 }
             } while (ret >= 0);
-            av_packet_unref(&avp);
+            av_packet_free(&avp);
             av_frame_free(&frame);
             parse_pos += parse_len;
         }
@@ -935,39 +944,9 @@ static void cc_send_to_decoder(hb_work_private_t *pv, hb_buffer_t *buf)
 
 static hb_buffer_t * cc_fill_buffer(hb_work_private_t *pv, uint8_t *cc, int size)
 {
-    int cc_count[4] = {0,};
-    int ii;
-    hb_buffer_t *buf = NULL;
+    hb_buffer_t * buf = hb_buffer_init(size);
 
-    for (ii = 0; ii < size; ii += 3)
-    {
-        if ((cc[ii] & 0x04) == 0)    // not valid
-            continue;
-        if ((cc[ii+1] & 0x7f) == 0 && (cc[ii+2] & 0x7f) == 0) // stuffing
-            continue;
-        int type = cc[ii] & 0x03;
-        cc_count[type]++;
-    }
-
-    // Only handles CC1 for now.
-    if (cc_count[0] > 0)
-    {
-        buf = hb_buffer_init(cc_count[0] * 2);
-        int jj = 0;
-        for (ii = 0; ii < size; ii += 3)
-        {
-            if ((cc[ii] & 0x04) == 0)    // not valid
-                continue;
-            if ((cc[ii+1] & 0x7f) == 0 && (cc[ii+2] & 0x7f) == 0) // stuffing
-                continue;
-            int type = cc[ii] & 0x03;
-            if (type == 0)
-            {
-                buf->data[jj++] = cc[ii+1];
-                buf->data[jj++] = cc[ii+2];
-            }
-        }
-    }
+    memcpy(buf->data, cc, size);
     return buf;
 }
 
@@ -984,7 +963,7 @@ static hb_buffer_t *copy_frame( hb_work_private_t *pv )
     if (pv->qsv.decode &&
         pv->qsv.config.io_pattern == MFX_IOPATTERN_OUT_VIDEO_MEMORY)
     {
-        out = hb_qsv_copy_frame(pv->frame, pv->job->qsv.ctx);
+        out = hb_qsv_copy_avframe_to_video_buffer(pv->job, pv->frame, 0);
     }
     else
 #endif
@@ -992,6 +971,14 @@ static hb_buffer_t *copy_frame( hb_work_private_t *pv )
         out = hb_avframe_to_video_buffer(pv->frame, (AVRational){1,1});
     }
 
+    // Make sure every frame is tagged.
+    if (out->f.color_prim == HB_COLR_PRI_UNDEF || out->f.color_transfer == HB_COLR_TRA_UNDEF || out->f.color_matrix == HB_COLR_MAT_UNDEF)
+    {
+        out->f.color_prim = pv->title->color_prim;
+        out->f.color_transfer = pv->title->color_transfer;
+        out->f.color_matrix = pv->title->color_matrix;
+    }
+
     if (pv->frame->pts != AV_NOPTS_VALUE)
     {
         reordered = reordered_hash_rem(pv, pv->frame->pts);
@@ -1074,12 +1061,15 @@ static hb_buffer_t *copy_frame( hb_work_private_t *pv )
 
                 subtitle        = calloc(sizeof( hb_subtitle_t ), 1);
                 subtitle->track = hb_list_count(pv->title->list_subtitle);
-                subtitle->id          = HB_SUBTITLE_EMBEDDED_CC_TAG;
-                subtitle->format      = TEXTSUB;
-                subtitle->source      = CC608SUB;
-                subtitle->config.dest = PASSTHRUSUB;
-                subtitle->codec       = WORK_DECCC608;
-                subtitle->attributes  = HB_SUBTITLE_ATTR_CC;
+                subtitle->id           = HB_SUBTITLE_EMBEDDED_CC_TAG;
+                subtitle->format       = TEXTSUB;
+                subtitle->source       = CC608SUB;
+                subtitle->config.dest  = PASSTHRUSUB;
+                subtitle->codec        = WORK_DECAVSUB;
+                subtitle->codec_param  = AV_CODEC_ID_EIA_608;
+                subtitle->attributes   = HB_SUBTITLE_ATTR_CC;
+                subtitle->timebase.num = 1;
+                subtitle->timebase.den = 90000;
 
                 /*
                  * The language of the subtitles will be the same as the
@@ -1110,12 +1100,36 @@ static hb_buffer_t *copy_frame( hb_work_private_t *pv )
             if (cc_buf != NULL)
             {
                 cc_buf->s.start        = out->s.start;
+                cc_buf->s.duration     = (int64_t)AV_NOPTS_VALUE;
                 cc_buf->s.scr_sequence = out->s.scr_sequence;
             }
             cc_send_to_decoder(pv, cc_buf);
         }
     }
 
+    // Check for HDR mastering data
+    sd = av_frame_get_side_data(pv->frame, AV_FRAME_DATA_MASTERING_DISPLAY_METADATA);
+    if (sd != NULL)
+    {
+        if (!pv->job && pv->title && sd->size > 0)
+        {
+            AVMasteringDisplayMetadata *mastering = (AVMasteringDisplayMetadata *)sd->data;
+            pv->title->mastering = hb_mastering_ff_to_hb(*mastering);
+        }
+    }
+
+    // Check for HDR content light level data
+    sd = av_frame_get_side_data(pv->frame, AV_FRAME_DATA_CONTENT_LIGHT_LEVEL);
+    if (sd != NULL)
+    {
+        if (!pv->job && pv->title && sd->size > 0)
+        {
+            AVContentLightMetadata *coll = (AVContentLightMetadata *)sd->data;
+            pv->title->coll.max_cll = coll->MaxCLL;
+            pv->title->coll.max_fall = coll->MaxFALL;
+        }
+    }
+
     return out;
 }
 
@@ -1127,6 +1141,7 @@ int reinit_video_filters(hb_work_private_t * pv)
     hb_dict_t        * settings;
     hb_filter_init_t   filter_init;
     enum AVPixelFormat pix_fmt;
+    enum AVColorRange  color_range;
 
 #if HB_PROJECT_FEATURE_QSV
     if (pv->qsv.decode &&
@@ -1144,6 +1159,7 @@ int reinit_video_filters(hb_work_private_t * pv)
         orig_width = pv->context->width & ~1;
         orig_height = pv->context->height & ~1;
         pix_fmt = AV_PIX_FMT_YUV420P;
+        color_range = AVCOL_RANGE_MPEG;
     }
     else
     {
@@ -1158,12 +1174,14 @@ int reinit_video_filters(hb_work_private_t * pv)
             orig_width = pv->job->title->geometry.width;
             orig_height = pv->job->title->geometry.height;
         }
-        pix_fmt = pv->job->pix_fmt;
+        pix_fmt = pv->job->input_pix_fmt;
+        color_range = pv->job->color_range;
     }
 
     if (pix_fmt            == pv->frame->format  &&
         orig_width         == pv->frame->width   &&
         orig_height        == pv->frame->height  &&
+        color_range        == pv->frame->color_range &&
         HB_ROTATION_0      == pv->title->rotation)
     {
         // No filtering required.
@@ -1171,18 +1189,20 @@ int reinit_video_filters(hb_work_private_t * pv)
         return 0;
     }
 
-    if (pv->video_filters.graph   != NULL              &&
-        pv->video_filters.width   == pv->frame->width  &&
-        pv->video_filters.height  == pv->frame->height &&
-        pv->video_filters.pix_fmt == pv->frame->format)
+    if (pv->video_filters.graph       != NULL              &&
+        pv->video_filters.width       == pv->frame->width  &&
+        pv->video_filters.height      == pv->frame->height &&
+        pv->video_filters.color_range == pv->frame->color_range &&
+        pv->video_filters.pix_fmt     == pv->frame->format)
     {
         // Current filter settings are good
         return 0;
     }
 
-    pv->video_filters.width   = pv->frame->width;
-    pv->video_filters.height  = pv->frame->height;
-    pv->video_filters.pix_fmt = pv->frame->format;
+    pv->video_filters.width       = pv->frame->width;
+    pv->video_filters.height      = pv->frame->height;
+    pv->video_filters.color_range = pv->frame->color_range;
+    pv->video_filters.pix_fmt     = pv->frame->format;
 
     // New filter required, create filter graph
     close_video_filters(pv);
@@ -1197,18 +1217,37 @@ int reinit_video_filters(hb_work_private_t * pv)
     filters = hb_value_array_init();
     if (pix_fmt            != pv->frame->format ||
         orig_width         != pv->frame->width  ||
-        orig_height        != pv->frame->height)
+        orig_height        != pv->frame->height ||
+        color_range        != pv->frame->color_range)
     {
-
         settings = hb_dict_init();
-        hb_dict_set(settings, "w", hb_value_int(orig_width));
-        hb_dict_set(settings, "h", hb_value_int(orig_height));
-        hb_dict_set(settings, "flags", hb_value_string("lanczos+accurate_rnd"));
-        hb_avfilter_append_dict(filters, "scale", settings);
+#if HB_PROJECT_FEATURE_QSV && (defined( _WIN32 ) || defined( __MINGW32__ ))
+        if (hb_qsv_hw_filters_are_enabled(pv->job))
+        {
+            hb_dict_set(settings, "w", hb_value_int(orig_width));
+            hb_dict_set(settings, "h", hb_value_int(orig_height));
+            hb_avfilter_append_dict(filters, "scale_qsv", settings);
+        }
+        else
+#endif
+        {
+            hb_dict_set(settings, "w", hb_value_int(orig_width));
+            hb_dict_set(settings, "h", hb_value_int(orig_height));
+            hb_dict_set(settings, "flags", hb_value_string("lanczos+accurate_rnd"));
+            if (color_range == AVCOL_RANGE_JPEG)
+            {
+                hb_dict_set_string(settings, "out_range", "full");
+            }
+            else
+            {
+                hb_dict_set_string(settings, "out_range", "limited");
+            }
+            hb_avfilter_append_dict(filters, "scale", settings);
 
-        settings = hb_dict_init();
-        hb_dict_set(settings, "pix_fmts", hb_value_string("yuv420p"));
-        hb_avfilter_append_dict(filters, "format", settings);
+            settings = hb_dict_init();
+            hb_dict_set(settings, "pix_fmts", hb_value_string(av_get_pix_fmt_name(pix_fmt)));
+            hb_avfilter_append_dict(filters, "format", settings);
+        }
     }
     if (pv->title->rotation != HB_ROTATION_0)
     {
@@ -1236,6 +1275,7 @@ int reinit_video_filters(hb_work_private_t * pv)
         }
     }
 
+    filter_init.job               = pv->job;
     filter_init.pix_fmt           = pv->frame->format;
     filter_init.geometry.width    = pv->frame->width;
     filter_init.geometry.height   = pv->frame->height;
@@ -1298,7 +1338,7 @@ static void filter_video(hb_work_private_t *pv)
 static int decodeFrame( hb_work_private_t * pv, packet_info_t * packet_info )
 {
     int got_picture = 0, oldlevel = 0, ret;
-    AVPacket avp;
+    AVPacket *avp = pv->pkt;
     reordered_data_t * reordered;
 
     if ( global_verbosity_level <= 1 )
@@ -1307,13 +1347,12 @@ static int decodeFrame( hb_work_private_t * pv, packet_info_t * packet_info )
         av_log_set_level( AV_LOG_QUIET );
     }
 
-    av_init_packet(&avp);
     if (packet_info != NULL)
     {
-        avp.data = packet_info->data;
-        avp.size = packet_info->size;
-        avp.pts  = pv->sequence;
-        avp.dts  = pv->sequence;
+        avp->data = packet_info->data;
+        avp->size = packet_info->size;
+        avp->pts  = pv->sequence;
+        avp->dts  = pv->sequence;
         reordered = malloc(sizeof(*reordered));
         if (reordered != NULL)
         {
@@ -1329,29 +1368,29 @@ static int decodeFrame( hb_work_private_t * pv, packet_info_t * packet_info )
         // PNG in a mov container.
         if (packet_info->frametype & HB_FRAME_MASK_KEY)
         {
-            avp.flags |= AV_PKT_FLAG_KEY;
+            avp->flags |= AV_PKT_FLAG_KEY;
         }
-        avp.flags  |= packet_info->discard * AV_PKT_FLAG_DISCARD;
+        avp->flags  |= packet_info->discard * AV_PKT_FLAG_DISCARD;
     }
     else
     {
-        avp.data = NULL;
-        avp.size = 0;
+        avp->data = NULL;
+        avp->size = 0;
     }
 
     if (pv->palette != NULL)
     {
         uint8_t * palette;
         int size;
-        palette = av_packet_new_side_data(&avp, AV_PKT_DATA_PALETTE,
+        palette = av_packet_new_side_data(avp, AV_PKT_DATA_PALETTE,
                                           AVPALETTE_SIZE);
         size = MIN(pv->palette->size, AVPALETTE_SIZE);
         memcpy(palette, pv->palette->data, size);
         hb_buffer_close(&pv->palette);
     }
 
-    ret = avcodec_send_packet(pv->context, &avp);
-    av_packet_unref(&avp);
+    ret = avcodec_send_packet(pv->context, avp);
+    av_packet_unref(avp);
     if (ret < 0 && ret != AVERROR_EOF)
     {
         ++pv->decode_errors;
@@ -1407,7 +1446,7 @@ static int decavcodecvInit( hb_work_object_t * w, hb_job_t * job )
         pv->qsv.config.io_pattern = MFX_IOPATTERN_OUT_SYSTEM_MEMORY;
         if(hb_qsv_full_path_is_enabled(job))
         {
-            hb_qsv_info_t *info = hb_qsv_info_get(job->vcodec);
+            hb_qsv_info_t *info = hb_qsv_encoder_info_get(hb_qsv_get_adapter_index(), job->vcodec);
             if (info != NULL)
             {
                 // setup the QSV configuration
@@ -1422,15 +1461,23 @@ static int decavcodecvInit( hb_work_object_t * w, hb_job_t * job )
                     // more surfaces may be needed for the lookahead
                     pv->qsv.config.additional_buffers = 160;
                 }
-                if(!pv->job->qsv.ctx)
+                if (!pv->job->qsv.ctx)
                 {
-                    pv->job->qsv.ctx = av_mallocz(sizeof(hb_qsv_context));
-                    if(!pv->job->qsv.ctx)
+                    hb_error( "decavcodecvInit: no context" );
+                    return 1;
+                }
+                pv->job->qsv.ctx->full_path_is_enabled = 1;
+                if (!pv->job->qsv.ctx->hb_dec_qsv_frames_ctx)
+                {
+                    pv->job->qsv.ctx->hb_dec_qsv_frames_ctx = av_mallocz(sizeof(HBQSVFramesContext));
+                    if(!pv->job->qsv.ctx->hb_dec_qsv_frames_ctx)
                     {
-                        hb_error( "decavcodecvInit: qsv ctx alloc failed" );
+                        hb_error( "decavcodecvInit: HBQSVFramesContext dec alloc failed" );
                         return 1;
                     }
-                    hb_qsv_add_context_usage(pv->job->qsv.ctx, 0);
+                }
+                if (!pv->job->qsv.ctx->dec_space)
+                {
                     pv->job->qsv.ctx->dec_space = av_mallocz(sizeof(hb_qsv_space));
                     if(!pv->job->qsv.ctx->dec_space)
                     {
@@ -1481,14 +1528,15 @@ static int decavcodecvInit( hb_work_object_t * w, hb_job_t * job )
         if (pv->qsv.decode &&
             pv->qsv.config.io_pattern == MFX_IOPATTERN_OUT_VIDEO_MEMORY)
         {
-            // assign callbacks
-            pv->context->get_format = hb_qsv_get_format;
-            pv->context->get_buffer2 = hb_qsv_get_buffer;
+            // assign callbacks and job to have access to qsv context from ffmpeg
+            pv->context->get_format      = hb_qsv_get_format;
+            pv->context->get_buffer2     = hb_qsv_get_buffer;
+            pv->context->opaque          = pv->job;
             pv->context->hwaccel_context = 0;
         }
 #endif
 
-        // Set encoder opts
+        // Set decoder opts
         AVDictionary * av_opts = NULL;
         av_dict_set( &av_opts, "refcounted_frames", "1", 0 );
         if (pv->title->flags & HBTF_NO_IDR)
@@ -1527,6 +1575,13 @@ static int decavcodecvInit( hb_work_object_t * w, hb_job_t * job )
         return 1;
     }
 
+    pv->pkt = av_packet_alloc();
+    if (pv->pkt == NULL)
+    {
+        hb_log("decavcodecvInit: av_packet_alloc failed");
+        return 1;
+    }
+
     /*
      * If not scanning, then are we supposed to extract Closed Captions
      * and send them to the decoder?
@@ -2107,23 +2162,10 @@ static int decavcodecvInfo( hb_work_object_t *w, hb_work_info_t *info )
     info->video_decode_support = HB_DECODE_SUPPORT_SW;
 
 #if HB_PROJECT_FEATURE_QSV
-    if (avcodec_find_decoder_by_name(hb_qsv_decode_get_codec_name(pv->context->codec_id)))
+    if (hb_qsv_available())
     {
-        switch (pv->context->codec_id)
-        {
-            case AV_CODEC_ID_HEVC:
-            case AV_CODEC_ID_H264:
-                if (pv->context->pix_fmt == AV_PIX_FMT_YUV420P  ||
-                    pv->context->pix_fmt == AV_PIX_FMT_YUVJ420P ||
-                    pv->context->pix_fmt == AV_PIX_FMT_YUV420P10LE)
-                {
-                    info->video_decode_support |= HB_DECODE_SUPPORT_QSV;
-                }
-                break;
-
-            default:
-                break;
-        }
+        if (hb_qsv_decode_codec_supported_codec(hb_qsv_get_adapter_index(), pv->context->codec_id, pv->context->pix_fmt))
+            info->video_decode_support |= HB_DECODE_SUPPORT_QSV;
     }
 #endif
 
@@ -2176,7 +2218,7 @@ hb_work_object_t hb_decavcodecv =
 static void decodeAudio(hb_work_private_t *pv, packet_info_t * packet_info)
 {
     AVCodecContext * context = pv->context;
-    AVPacket         avp;
+    AVPacket       * avp = pv->pkt;
     int              ret;
 
     // libav does not supply timestamps for wmapro audio (possibly others)
@@ -2187,25 +2229,24 @@ static void decodeAudio(hb_work_private_t *pv, packet_info_t * packet_info)
     {
         pv->next_pts = packet_info->pts;
     }
-    av_init_packet(&avp);
     if (packet_info != NULL)
     {
-        avp.data = packet_info->data;
-        avp.size = packet_info->size;
-        avp.pts  = packet_info->pts;
-        avp.dts  = AV_NOPTS_VALUE;
-        avp.flags |= packet_info->discard * AV_PKT_FLAG_DISCARD;
+        avp->data = packet_info->data;
+        avp->size = packet_info->size;
+        avp->pts  = packet_info->pts;
+        avp->dts  = AV_NOPTS_VALUE;
+        avp->flags |= packet_info->discard * AV_PKT_FLAG_DISCARD;
     }
     else
     {
-        avp.data = NULL;
-        avp.size = 0;
+        avp->data = NULL;
+        avp->size = 0;
     }
 
-    ret = avcodec_send_packet(context, &avp);
+    ret = avcodec_send_packet(context, avp);
     if (ret < 0 && ret != AVERROR_EOF)
     {
-        av_packet_unref(&avp);
+        av_packet_unref(avp);
         return;
     }
 
@@ -2243,8 +2284,8 @@ static void decodeAudio(hb_work_private_t *pv, packet_info_t * packet_info)
             // Note that even though we are doing passthru, we had to decode
             // so that we know the stop time and the pts of the next audio
             // packet.
-            out = hb_buffer_init(avp.size);
-            memcpy(out->data, avp.data, avp.size);
+            out = hb_buffer_init(avp->size);
+            memcpy(out->data, avp->data, avp->size);
         }
         else
         {
@@ -2289,7 +2330,7 @@ static void decodeAudio(hb_work_private_t *pv, packet_info_t * packet_info)
             {
                 hb_log("decavcodec: hb_audio_resample_update() failed");
                 av_frame_unref(pv->frame);
-                av_packet_unref(&avp);
+                av_packet_unref(avp);
                 return;
             }
             out = hb_audio_resample(pv->resample,
@@ -2344,5 +2385,5 @@ static void decodeAudio(hb_work_private_t *pv, packet_info_t * packet_info)
         av_frame_unref(pv->frame);
         ++pv->nframes;
     } while (ret >= 0);
-    av_packet_unref(&avp);
+    av_packet_unref(avp);
 }
