@@ -1,3 +1,4 @@
+
 /* ********************************************************************* *\
 
 Copyright (C) 2013 Intel Corporation.  All rights reserved.
@@ -42,9 +43,6 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "libavutil/hwcontext.h"
 #include <mfx/mfxvideo.h>
 
-extern AVBufferRef *hb_hw_device_ctx;
-EncQSVFramesContext hb_enc_qsv_frames_ctx;
-
 /*
  * The frame info struct remembers information about each frame across calls to
  * the encoder. Since frames are uniquely identified by their timestamp, we use
@@ -507,8 +505,7 @@ AVBufferRef *hb_qsv_create_mids(AVBufferRef *hw_frames_ref)
 
     for (i = 0; i < nb_surfaces; i++) {
         QSVMid *mid = &mids[i];
-        mid->handle        = frames_hwctx->surfaces[i].Data.MemId;
-        mid->texture       = frames_hwctx->texture;
+        mid->handle_pair   = (mfxHDLPair*)frames_hwctx->surfaces[i].Data.MemId;
         mid->hw_frames_ref = hw_frames_ref1;
     }
 
@@ -554,7 +551,7 @@ static int qsv_setup_mids(mfxFrameAllocResponse *resp, AVBufferRef *hw_frames_re
 static mfxStatus hb_qsv_frame_alloc(mfxHDL pthis, mfxFrameAllocRequest *req,
                                  mfxFrameAllocResponse *resp)
 {
-    EncQSVFramesContext *ctx = pthis;
+    HBQSVFramesContext *ctx = pthis;
     int ret;
 
     /* this should only be called from an encoder or decoder and
@@ -682,7 +679,7 @@ static mfxStatus hb_qsv_frame_lock(mfxHDL pthis, mfxMemId mid, mfxFrameData *ptr
         goto fail;
 
     qsv_mid->surf.Info = hw_frames_hwctx->surfaces[0].Info;
-    qsv_mid->surf.Data.MemId = qsv_mid->handle;
+    qsv_mid->surf.Data.MemId = qsv_mid->handle_pair;
 
     /* map the data to the system memory */
     ret = av_hwframe_map(qsv_mid->locked_frame, qsv_mid->hw_frame,
@@ -715,14 +712,12 @@ static mfxStatus hb_qsv_frame_unlock(mfxHDL pthis, mfxMemId mid, mfxFrameData *p
 static mfxStatus hb_qsv_frame_get_hdl(mfxHDL pthis, mfxMemId mid, mfxHDL *hdl)
 {
     QSVMid *qsv_mid = (QSVMid*)mid;
+    mfxHDLPair *pair_dst = (mfxHDLPair*)hdl;
+    mfxHDLPair *pair_src = (mfxHDLPair*)qsv_mid->handle_pair;
 
-    if (qsv_mid->texture) {
-        mfxHDLPair *pPair  =  (mfxHDLPair*)hdl;
-        pPair->first  = qsv_mid->texture;
-        pPair->second = qsv_mid->handle;
-    } else {
-        *hdl = qsv_mid->handle;
-    }
+    pair_dst->first = pair_src->first;
+    if (pair_src->second != (mfxMemId)MFX_INFINITE)
+        pair_dst->second = pair_src->second;
 
     return MFX_ERR_NONE;
 }
@@ -747,12 +742,8 @@ int qsv_enc_init(hb_work_private_t *pv)
 
     if (qsv == NULL)
     {
-        if (!pv->is_sys_mem)
-        {
-            hb_error("qsv_enc_init: decode enabled but no context!");
-            return 3;
-        }
-        job->qsv.ctx = qsv = av_mallocz(sizeof(hb_qsv_context));
+        hb_error("qsv_enc_init: no context!");
+        return 3;
     }
 
     hb_qsv_space *qsv_encode = qsv->enc_space;
@@ -761,9 +752,6 @@ int qsv_enc_init(hb_work_private_t *pv)
         // if only for encode
         if (pv->is_sys_mem)
         {
-            // no need to use additional sync as encode only -> single thread
-            hb_qsv_add_context_usage(qsv, 0);
-
             // re-use the session from encqsvInit
             qsv->mfx_session = pv->mfx_session;
         }
@@ -774,7 +762,7 @@ int qsv_enc_init(hb_work_private_t *pv)
             mfxVersion    ver;
             mfxIMPL       impl;
 
-            AVHWDeviceContext    *device_ctx = (AVHWDeviceContext*)hb_hw_device_ctx->data;
+            AVHWDeviceContext    *device_ctx = (AVHWDeviceContext*)qsv->hb_hw_device_ctx->data;
             AVQSVDeviceContext *device_hwctx = device_ctx->hwctx;
             mfxSession        parent_session = device_hwctx->session;
 
@@ -795,7 +783,7 @@ int qsv_enc_init(hb_work_private_t *pv)
             // reuse parent session
             qsv->mfx_session = parent_session;
             mfxFrameAllocator frame_allocator = {
-                .pthis  = &hb_enc_qsv_frames_ctx,
+                .pthis  = pv->job->qsv.ctx->hb_dec_qsv_frames_ctx,
                 .Alloc  = hb_qsv_frame_alloc,
                 .Lock   = hb_qsv_frame_lock,
                 .Unlock = hb_qsv_frame_unlock,
@@ -803,6 +791,11 @@ int qsv_enc_init(hb_work_private_t *pv)
                 .Free   = hb_qsv_frame_free,
             };
 
+            if (hb_qsv_hw_filters_are_enabled(pv->job))
+            {
+                frame_allocator.pthis = pv->job->qsv.ctx->hb_vpp_qsv_frames_ctx;
+            }
+
             err = MFXVideoCORE_SetFrameAllocator(qsv->mfx_session, &frame_allocator);
             if (err != MFX_ERR_NONE)
             {
@@ -858,9 +851,9 @@ int qsv_enc_init(hb_work_private_t *pv)
         {
             pv->sws_context_to_nv12 = hb_sws_get_context(
                                         job->width, job->height,
-                                        AV_PIX_FMT_YUV420P,
+                                        job->output_pix_fmt, job->color_range,
                                         job->width, job->height,
-                                        AV_PIX_FMT_P010LE,
+                                        AV_PIX_FMT_P010LE, job->color_range,
                                         SWS_LANCZOS|SWS_ACCURATE_RND,
                                         SWS_CS_DEFAULT);
         }
@@ -868,9 +861,9 @@ int qsv_enc_init(hb_work_private_t *pv)
         {
             pv->sws_context_to_nv12 = hb_sws_get_context(
                                         job->width, job->height,
-                                        AV_PIX_FMT_YUV420P,
+                                        job->output_pix_fmt, job->color_range,
                                         job->width, job->height,
-                                        AV_PIX_FMT_NV12,
+                                        AV_PIX_FMT_NV12, job->color_range,
                                         SWS_LANCZOS|SWS_ACCURATE_RND,
                                         SWS_CS_DEFAULT);
         }
@@ -900,7 +893,7 @@ int qsv_enc_init(hb_work_private_t *pv)
             *job->die = 1;
             return -1;
         }
-        pv->loaded_plugins = hb_qsv_load_plugins(pv->qsv_info, qsv->mfx_session, version);
+        pv->loaded_plugins = hb_qsv_load_plugins(hb_qsv_get_adapter_index(), pv->qsv_info, qsv->mfx_session, version);
         if (pv->loaded_plugins == NULL)
         {
             hb_error("qsv_enc_init: hb_qsv_load_plugins failed");
@@ -1006,6 +999,199 @@ int qsv_enc_init(hb_work_private_t *pv)
     return 0;
 }
 
+static int log_encoder_params(const hb_work_private_t *pv, const mfxVideoParam *videoParam)
+{
+    const mfxExtCodingOption  *option1 = NULL;
+    const mfxExtCodingOption2 *option2 = NULL;
+    const mfxExtVideoSignalInfo *extVideoSignalInfo = NULL;
+
+    // log code path and main output settings
+    hb_log("encqsvInit: using %s%s path",
+           pv->is_sys_mem ? "encode-only" : "full QSV",
+           videoParam->mfx.LowPower == MFX_CODINGOPTION_ON ? " (LowPower)" : "" );
+    hb_log("encqsvInit: %s %s profile @ level %s",
+           hb_qsv_codec_name  (videoParam->mfx.CodecId),
+           hb_qsv_profile_name(videoParam->mfx.CodecId, videoParam->mfx.CodecProfile),
+           hb_qsv_level_name  (videoParam->mfx.CodecId, videoParam->mfx.CodecLevel));
+    hb_log("encqsvInit: TargetUsage %"PRIu16" AsyncDepth %"PRIu16"",
+           videoParam->mfx.TargetUsage, videoParam->AsyncDepth);
+    hb_log("encqsvInit: GopRefDist %"PRIu16" GopPicSize %"PRIu16" NumRefFrame %"PRIu16"",
+           videoParam->mfx.GopRefDist, videoParam->mfx.GopPicSize, videoParam->mfx.NumRefFrame);
+
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_B_REF_PYRAMID)
+    {
+        hb_log("encqsvInit: BFramesMax %d BRefType %s",
+               videoParam->mfx.GopRefDist > 1 ?
+               videoParam->mfx.GopRefDist - 1 : 0,
+               pv->param.gop.b_pyramid ? "pyramid" : "off");
+    }
+    else
+    {
+        hb_log("encqsvInit: BFramesMax %d",
+               videoParam->mfx.GopRefDist > 1 ?
+               videoParam->mfx.GopRefDist - 1 : 0);
+    }
+
+    for (int i = 0; i < videoParam->NumExtParam; i++)
+    {
+        mfxExtCodingOption *option = (mfxExtCodingOption*)videoParam->ExtParam[i];
+        if (option->Header.BufferId == MFX_EXTBUFF_VIDEO_SIGNAL_INFO)
+        {
+            extVideoSignalInfo = (mfxExtVideoSignalInfo*)videoParam->ExtParam[i];
+        }
+        else if (option->Header.BufferId == MFX_EXTBUFF_CODING_OPTION)
+        {
+            option1 = (mfxExtCodingOption*)videoParam->ExtParam[i];
+        }
+        else if (option->Header.BufferId == MFX_EXTBUFF_CODING_OPTION2)
+        {
+            option2 = (mfxExtCodingOption2*)videoParam->ExtParam[i];
+        }
+        else
+        {
+            hb_log("Unknown Header.BufferId=%d", option->Header.BufferId);
+        }
+    }
+
+    if (option2 && (option2->AdaptiveI != MFX_CODINGOPTION_OFF ||
+        option2->AdaptiveB != MFX_CODINGOPTION_OFF))
+    {
+        if (videoParam->mfx.GopRefDist > 1)
+        {
+            hb_log("encqsvInit: AdaptiveI %s AdaptiveB %s",
+                hb_qsv_codingoption_get_name(option2->AdaptiveI),
+                hb_qsv_codingoption_get_name(option2->AdaptiveB));
+        }
+        else
+        {
+            hb_log("encqsvInit: AdaptiveI %s",
+                hb_qsv_codingoption_get_name(option2->AdaptiveI));
+        }
+    }
+
+    if (videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CQP)
+    {
+        char qpi[7], qpp[9], qpb[9];
+        snprintf(qpi, sizeof(qpi),  "QPI %"PRIu16"", videoParam->mfx.QPI);
+        snprintf(qpp, sizeof(qpp), " QPP %"PRIu16"", videoParam->mfx.QPP);
+        snprintf(qpb, sizeof(qpb), " QPB %"PRIu16"", videoParam->mfx.QPB);
+        hb_log("encqsvInit: RateControlMethod CQP with %s%s%s", qpi,
+               videoParam->mfx.GopPicSize > 1 ? qpp : "",
+               videoParam->mfx.GopRefDist > 1 ? qpb : "");
+    }
+    else
+    {
+        switch (videoParam->mfx.RateControlMethod)
+        {
+            case MFX_RATECONTROL_LA:
+                hb_log("encqsvInit: RateControlMethod LA TargetKbps %"PRIu16" BRCParamMultiplier %"PRIu16" LookAheadDepth %"PRIu16"",
+                       videoParam->mfx.TargetKbps, videoParam->mfx.BRCParamMultiplier, (option2 != NULL) ? option2->LookAheadDepth : 0);
+                break;
+            case MFX_RATECONTROL_LA_ICQ:
+                hb_log("encqsvInit: RateControlMethod LA_ICQ ICQQuality %"PRIu16" LookAheadDepth %"PRIu16"",
+                       videoParam->mfx.ICQQuality, (option2 != NULL) ? option2->LookAheadDepth : 0);
+                break;
+            case MFX_RATECONTROL_ICQ:
+                hb_log("encqsvInit: RateControlMethod ICQ ICQQuality %"PRIu16"",
+                       videoParam->mfx.ICQQuality);
+                break;
+            case MFX_RATECONTROL_CBR:
+            case MFX_RATECONTROL_VBR:
+                hb_log("encqsvInit: RateControlMethod %s TargetKbps %"PRIu16" MaxKbps %"PRIu16" BufferSizeInKB %"PRIu16" InitialDelayInKB %"PRIu16" BRCParamMultiplier %"PRIu16"",
+                       videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CBR ? "CBR" : "VBR",
+                       videoParam->mfx.TargetKbps,     videoParam->mfx.MaxKbps,
+                       videoParam->mfx.BufferSizeInKB, videoParam->mfx.InitialDelayInKB, videoParam->mfx.BRCParamMultiplier);
+                break;
+            default:
+                hb_log("encqsvInit: invalid rate control method %"PRIu16"",
+                       videoParam->mfx.RateControlMethod);
+                return -1;
+        }
+    }
+
+    if (option2 && (videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA ||
+        videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA_ICQ))
+    {
+        switch (option2->LookAheadDS)
+        {
+            case MFX_LOOKAHEAD_DS_UNKNOWN:
+                hb_log("encqsvInit: LookAheadDS unknown (auto)");
+                break;
+            case MFX_LOOKAHEAD_DS_OFF: // default
+                break;
+            case MFX_LOOKAHEAD_DS_2x:
+                hb_log("encqsvInit: LookAheadDS 2x");
+                break;
+            case MFX_LOOKAHEAD_DS_4x:
+                hb_log("encqsvInit: LookAheadDS 4x");
+                break;
+            default:
+                hb_log("encqsvInit: invalid LookAheadDS value 0x%"PRIx16"",
+                       option2->LookAheadDS);
+                break;
+        }
+    }
+
+    switch (videoParam->mfx.FrameInfo.PicStruct)
+    {
+        case MFX_PICSTRUCT_PROGRESSIVE: // default
+            break;
+        case MFX_PICSTRUCT_FIELD_TFF:
+            hb_log("encqsvInit: PicStruct top field first");
+            break;
+        case MFX_PICSTRUCT_FIELD_BFF:
+            hb_log("encqsvInit: PicStruct bottom field first");
+            break;
+        default:
+            hb_error("encqsvInit: invalid PicStruct value 0x%"PRIx16"",
+                     videoParam->mfx.FrameInfo.PicStruct);
+            return -1;
+    }
+
+    if (videoParam->mfx.CodecId == MFX_CODEC_AVC)
+    {
+        if (option1 && (option1->CAVLC != MFX_CODINGOPTION_OFF))
+        {
+            hb_log("encqsvInit: CAVLC %s",
+                hb_qsv_codingoption_get_name(option1->CAVLC));
+        }
+    }
+
+    if (option2 && (option2->ExtBRC != MFX_CODINGOPTION_OFF))
+    {
+        hb_log("encqsvInit: ExtBRC %s",
+            hb_qsv_codingoption_get_name(option2->ExtBRC));
+    }
+
+    if (option2 && (option2->MBBRC != MFX_CODINGOPTION_OFF))
+    {
+        hb_log("encqsvInit: MBBRC %s",
+            hb_qsv_codingoption_get_name(option2->MBBRC));
+    }
+
+    if (option2)
+    {
+        switch (option2->Trellis)
+        {
+            case MFX_TRELLIS_OFF: // default
+                break;
+            case MFX_TRELLIS_UNKNOWN:
+                hb_log("encqsvInit: Trellis unknown (auto)");
+                break;
+            default:
+                hb_log("encqsvInit: Trellis on (%s%s%s)",
+                       (option2->Trellis & MFX_TRELLIS_I) ? "I" : "",
+                       (option2->Trellis & MFX_TRELLIS_P) &&
+                       (videoParam->mfx.GopPicSize > 1)    ? "P" : "",
+                       (option2->Trellis & MFX_TRELLIS_B) &&
+                       (videoParam->mfx.GopRefDist > 1)    ? "B" : "");
+                break;
+        }
+    }
+
+    return 0;
+}
+
 /***********************************************************************
  * encqsvInit
  ***********************************************************************
@@ -1013,12 +1199,13 @@ int qsv_enc_init(hb_work_private_t *pv)
  **********************************************************************/
 int encqsvInit(hb_work_object_t *w, hb_job_t *job)
 {
+    int brc_param_multiplier;
     hb_work_private_t *pv = calloc(1, sizeof(hb_work_private_t));
     w->private_data       = pv;
 
     pv->is_sys_mem         = hb_qsv_full_path_is_enabled(job) ? 0 : 1; // TODO: re-implement QSV VPP filtering support
     pv->job                = job;
-    pv->qsv_info           = hb_qsv_info_get(job->vcodec);
+    pv->qsv_info           = hb_qsv_encoder_info_get(hb_qsv_get_adapter_index(), job->vcodec);
     pv->delayed_processing = hb_list_init();
     pv->last_start         = INT64_MIN;
     hb_buffer_list_clear(&pv->encoded_frames);
@@ -1057,7 +1244,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
             hb_value_t *value = hb_dict_iter_value(iter);
             char *str = hb_value_get_string_xform(value);
 
-            switch (hb_qsv_param_parse(&pv->param, pv->qsv_info, key, str))
+            switch (hb_qsv_param_parse(&pv->param, pv->qsv_info, pv->job, key, str))
             {
                 case HB_QSV_PARAM_OK:
                     break;
@@ -1083,7 +1270,16 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         }
         hb_dict_free(&options_list);
     }
-
+#if defined(_WIN32) || defined(__MINGW32__)
+    if (pv->is_sys_mem && hb_qsv_implementation_is_hardware(pv->qsv_info->implementation))
+    {
+        // select the right hardware implementation based on dx index
+        if (!job->qsv.ctx->qsv_device)
+            hb_qsv_param_parse_dx_index(pv->job, hb_qsv_get_adapter_index());
+        mfxIMPL hw_preference = MFX_IMPL_VIA_D3D11;
+        pv->qsv_info->implementation = hb_qsv_dx_index_to_impl(job->qsv.ctx->dx_index) | hw_preference;
+    }
+#endif
     // reload colorimetry in case values were set in encoder_options
     if (pv->param.videoSignalInfo.ColourDescriptionPresent)
     {
@@ -1219,26 +1415,36 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     {
         pv->param.rc.lookahead = pv->param.rc.lookahead && (pv->param.rc.icq || job->vquality <= HB_INVALID_VIDEO_QUALITY);
     }
-
+#if HB_PROJECT_FEATURE_QSV
+    if (pv->job->qsv.ctx != NULL)
+    {
+        job->qsv.ctx->la_is_enabled = pv->param.rc.lookahead ? 1 : 0;
+    }
+#endif
+    //libmfx BRC parameters are 16 bits thus maybe overflow, then BRCParamMultiplier is needed
+    brc_param_multiplier = (FFMAX(FFMAX3(job->vbitrate, pv->param.rc.vbv_max_bitrate, pv->param.rc.vbv_buffer_size / 8),
+                            pv->param.rc.vbv_buffer_init / 8) + 0x10000) / 0x10000;
     // set VBV here (this will be overridden for CQP and ignored for LA)
     // only set BufferSizeInKB, InitialDelayInKB and MaxKbps if we have
-    // them - otheriwse Media SDK will pick values for us automatically
+    // them - otherwise Media SDK will pick values for us automatically
     if (pv->param.rc.vbv_buffer_size > 0)
     {
         if (pv->param.rc.vbv_buffer_init > 1.0)
         {
-            pv->param.videoParam->mfx.InitialDelayInKB = (pv->param.rc.vbv_buffer_init / 8);
+            pv->param.videoParam->mfx.InitialDelayInKB = (pv->param.rc.vbv_buffer_init / 8) / brc_param_multiplier;
         }
         else if (pv->param.rc.vbv_buffer_init > 0.0)
         {
             pv->param.videoParam->mfx.InitialDelayInKB = (pv->param.rc.vbv_buffer_size *
-                                                          pv->param.rc.vbv_buffer_init / 8);
+                                                          pv->param.rc.vbv_buffer_init / 8) / brc_param_multiplier;
         }
-        pv->param.videoParam->mfx.BufferSizeInKB = (pv->param.rc.vbv_buffer_size / 8);
+        pv->param.videoParam->mfx.BufferSizeInKB       = (pv->param.rc.vbv_buffer_size / 8) / brc_param_multiplier;
+        pv->param.videoParam->mfx.BRCParamMultiplier   = brc_param_multiplier;
     }
     if (pv->param.rc.vbv_max_bitrate > 0)
     {
-        pv->param.videoParam->mfx.MaxKbps = pv->param.rc.vbv_max_bitrate;
+        pv->param.videoParam->mfx.MaxKbps              = pv->param.rc.vbv_max_bitrate / brc_param_multiplier;
+        pv->param.videoParam->mfx.BRCParamMultiplier   = brc_param_multiplier;
     }
 
     // set rate control parameters
@@ -1282,8 +1488,9 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         if (pv->param.rc.lookahead)
         {
             // introduced in API 1.7
-            pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_LA;
-            pv->param.videoParam->mfx.TargetKbps        = job->vbitrate;
+            pv->param.videoParam->mfx.RateControlMethod  = MFX_RATECONTROL_LA;
+            pv->param.videoParam->mfx.TargetKbps         = job->vbitrate / brc_param_multiplier;
+            pv->param.videoParam->mfx.BRCParamMultiplier = brc_param_multiplier;
             // ignored, but some drivers will change AsyncDepth because of it
             pv->param.codingOption2.ExtBRC = MFX_CODINGOPTION_OFF;
         }
@@ -1298,7 +1505,8 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
             {
                 pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_VBR;
             }
-            pv->param.videoParam->mfx.TargetKbps = job->vbitrate;
+            pv->param.videoParam->mfx.TargetKbps            = job->vbitrate / brc_param_multiplier;
+            pv->param.videoParam->mfx.BRCParamMultiplier    = brc_param_multiplier;
         }
     }
     else
@@ -1391,7 +1599,8 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
      * this is fine since the actual encode will use the same
      * values for all parameters relevant to the output bitstream
      */
-    mfxStatus err;
+    int err;
+    mfxStatus sts;
     mfxVersion version;
     mfxVideoParam videoParam;
     mfxExtBuffer *extParamArray[3];
@@ -1401,17 +1610,17 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     mfxExtCodingOptionSPSPPS sps_pps_buf, *sps_pps = &sps_pps_buf;
     version.Major = HB_QSV_MINVERSION_MAJOR;
     version.Minor = HB_QSV_MINVERSION_MINOR;
-    err = MFXInit(pv->qsv_info->implementation, &version, &session);
-    if (err != MFX_ERR_NONE)
+    sts = MFXInit(pv->qsv_info->implementation, &version, &session);
+    if (sts != MFX_ERR_NONE)
     {
-        hb_error("encqsvInit: MFXInit failed (%d)", err);
+        hb_error("encqsvInit: MFXInit failed (%d) with implementation %d", sts, pv->qsv_info->implementation);
         return -1;
     }
 
     if (pv->qsv_info->implementation & MFX_IMPL_HARDWARE_ANY)
     {
         // On linux, the handle to the VA display must be set.
-        // This code is essentiall a NOP other platforms.
+        // This code is essentially a NOP other platforms.
         pv->display = hb_qsv_display_init();
         if (pv->display != NULL)
         {
@@ -1421,16 +1630,16 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     }
 
     /* Query the API version for hb_qsv_load_plugins */
-    err = MFXQueryVersion(session, &version);
-    if (err != MFX_ERR_NONE)
+    sts = MFXQueryVersion(session, &version);
+    if (sts != MFX_ERR_NONE)
     {
-        hb_error("encqsvInit: MFXQueryVersion failed (%d)", err);
+        hb_error("encqsvInit: MFXQueryVersion failed (%d)", sts);
         MFXClose(session);
         return -1;
     }
 
     /* Load required MFX plug-ins */
-    pv->loaded_plugins = hb_qsv_load_plugins(pv->qsv_info, session, version);
+    pv->loaded_plugins = hb_qsv_load_plugins(hb_qsv_get_adapter_index(), pv->qsv_info, session, version);
     if (pv->loaded_plugins == NULL)
     {
         hb_error("encqsvInit: hb_qsv_load_plugins failed");
@@ -1438,22 +1647,27 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         return -1;
     }
 
-    err = MFXVideoENCODE_Init(session, pv->param.videoParam);
+    sts = MFXVideoENCODE_Init(session, pv->param.videoParam);
 // workaround for the early 15.33.x driver, should be removed later
 #define HB_DRIVER_FIX_33
 #ifdef  HB_DRIVER_FIX_33
     int la_workaround = 0;
-    if (err < MFX_ERR_NONE &&
+    if (sts < MFX_ERR_NONE &&
         pv->param.videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA)
     {
         pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_CBR;
-        err = MFXVideoENCODE_Init(session, pv->param.videoParam);
+        sts = MFXVideoENCODE_Init(session, pv->param.videoParam);
         la_workaround = 1;
     }
 #endif
-    if (err < MFX_ERR_NONE) // ignore warnings
+    if (sts < MFX_ERR_NONE) // ignore warnings
     {
-        hb_error("encqsvInit: MFXVideoENCODE_Init failed (%d)", err);
+        hb_error("encqsvInit: MFXVideoENCODE_Init failed (%d)", sts);
+        err = log_encoder_params(pv, pv->param.videoParam);
+        if (err < 0)
+        {
+            hb_error("encqsvInit: log_encoder_params failed (%d)", err);
+        }
         hb_qsv_unload_plugins(&pv->loaded_plugins, session, version);
         MFXClose(session);
         return -1;
@@ -1491,10 +1705,10 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     {
         videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)option2;
     }
-    err = MFXVideoENCODE_GetVideoParam(session, &videoParam);
-    if (err != MFX_ERR_NONE)
+    sts = MFXVideoENCODE_GetVideoParam(session, &videoParam);
+    if (sts != MFX_ERR_NONE)
     {
-        hb_error("encqsvInit: MFXVideoENCODE_GetVideoParam failed (%d)", err);
+        hb_error("encqsvInit: MFXVideoENCODE_GetVideoParam failed (%d)", sts);
         hb_qsv_unload_plugins(&pv->loaded_plugins, session, version);
         MFXClose(session);
         return -1;
@@ -1568,173 +1782,12 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         pv->list_dts = hb_list_init();
     }
 
-    // log code path and main output settings
-    hb_log("encqsvInit: using %s%s path",
-           pv->is_sys_mem ? "encode-only" : "full QSV",
-           videoParam.mfx.LowPower == MFX_CODINGOPTION_ON ? " (LowPower)" : "" );
-    hb_log("encqsvInit: %s %s profile @ level %s",
-           hb_qsv_codec_name  (videoParam.mfx.CodecId),
-           hb_qsv_profile_name(videoParam.mfx.CodecId, videoParam.mfx.CodecProfile),
-           hb_qsv_level_name  (videoParam.mfx.CodecId, videoParam.mfx.CodecLevel));
-    hb_log("encqsvInit: TargetUsage %"PRIu16" AsyncDepth %"PRIu16"",
-           videoParam.mfx.TargetUsage, videoParam.AsyncDepth);
-    hb_log("encqsvInit: GopRefDist %"PRIu16" GopPicSize %"PRIu16" NumRefFrame %"PRIu16"",
-           videoParam.mfx.GopRefDist, videoParam.mfx.GopPicSize, videoParam.mfx.NumRefFrame);
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_B_REF_PYRAMID)
+    err = log_encoder_params(pv, &videoParam);
+    if (err < 0)
     {
-        hb_log("encqsvInit: BFramesMax %d BRefType %s",
-               videoParam.mfx.GopRefDist > 1 ?
-               videoParam.mfx.GopRefDist - 1 : 0,
-               pv->param.gop.b_pyramid ? "pyramid" : "off");
+        hb_error("encqsvInit: log_encoder_params failed (%d)", err);
+        return -1;
     }
-    else
-    {
-        hb_log("encqsvInit: BFramesMax %d",
-               videoParam.mfx.GopRefDist > 1 ?
-               videoParam.mfx.GopRefDist - 1 : 0);
-    }
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_OPTION2_IB_ADAPT)
-    {
-        if (option2->AdaptiveI != MFX_CODINGOPTION_OFF ||
-            option2->AdaptiveB != MFX_CODINGOPTION_OFF)
-        {
-            if (videoParam.mfx.GopRefDist > 1)
-            {
-                hb_log("encqsvInit: AdaptiveI %s AdaptiveB %s",
-                       hb_qsv_codingoption_get_name(option2->AdaptiveI),
-                       hb_qsv_codingoption_get_name(option2->AdaptiveB));
-            }
-            else
-            {
-                hb_log("encqsvInit: AdaptiveI %s",
-                       hb_qsv_codingoption_get_name(option2->AdaptiveI));
-            }
-        }
-    }
-    if (videoParam.mfx.RateControlMethod == MFX_RATECONTROL_CQP)
-    {
-        char qpi[7], qpp[9], qpb[9];
-        snprintf(qpi, sizeof(qpi),  "QPI %"PRIu16"", videoParam.mfx.QPI);
-        snprintf(qpp, sizeof(qpp), " QPP %"PRIu16"", videoParam.mfx.QPP);
-        snprintf(qpb, sizeof(qpb), " QPB %"PRIu16"", videoParam.mfx.QPB);
-        hb_log("encqsvInit: RateControlMethod CQP with %s%s%s", qpi,
-               videoParam.mfx.GopPicSize > 1 ? qpp : "",
-               videoParam.mfx.GopRefDist > 1 ? qpb : "");
-    }
-    else
-    {
-        switch (videoParam.mfx.RateControlMethod)
-        {
-            case MFX_RATECONTROL_LA:
-                hb_log("encqsvInit: RateControlMethod LA TargetKbps %"PRIu16" LookAheadDepth %"PRIu16"",
-                       videoParam.mfx.TargetKbps, option2->LookAheadDepth);
-                break;
-            case MFX_RATECONTROL_LA_ICQ:
-                hb_log("encqsvInit: RateControlMethod LA_ICQ ICQQuality %"PRIu16" LookAheadDepth %"PRIu16"",
-                       videoParam.mfx.ICQQuality, option2->LookAheadDepth);
-                break;
-            case MFX_RATECONTROL_ICQ:
-                hb_log("encqsvInit: RateControlMethod ICQ ICQQuality %"PRIu16"",
-                       videoParam.mfx.ICQQuality);
-                break;
-            case MFX_RATECONTROL_CBR:
-            case MFX_RATECONTROL_VBR:
-                hb_log("encqsvInit: RateControlMethod %s TargetKbps %"PRIu16" MaxKbps %"PRIu16" BufferSizeInKB %"PRIu16" InitialDelayInKB %"PRIu16"",
-                       videoParam.mfx.RateControlMethod == MFX_RATECONTROL_CBR ? "CBR" : "VBR",
-                       videoParam.mfx.TargetKbps,     videoParam.mfx.MaxKbps,
-                       videoParam.mfx.BufferSizeInKB, videoParam.mfx.InitialDelayInKB);
-                break;
-            default:
-                hb_log("encqsvInit: invalid rate control method %"PRIu16"",
-                       videoParam.mfx.RateControlMethod);
-                return -1;
-        }
-    }
-    if ((pv->qsv_info->capabilities & HB_QSV_CAP_OPTION2_LA_DOWNS) &&
-        (videoParam.mfx.RateControlMethod == MFX_RATECONTROL_LA ||
-         videoParam.mfx.RateControlMethod == MFX_RATECONTROL_LA_ICQ))
-    {
-        switch (option2->LookAheadDS)
-        {
-            case MFX_LOOKAHEAD_DS_UNKNOWN:
-                hb_log("encqsvInit: LookAheadDS unknown (auto)");
-                break;
-            case MFX_LOOKAHEAD_DS_OFF: // default
-                break;
-            case MFX_LOOKAHEAD_DS_2x:
-                hb_log("encqsvInit: LookAheadDS 2x");
-                break;
-            case MFX_LOOKAHEAD_DS_4x:
-                hb_log("encqsvInit: LookAheadDS 4x");
-                break;
-            default:
-                hb_log("encqsvInit: invalid LookAheadDS value 0x%"PRIx16"",
-                       option2->LookAheadDS);
-                break;
-        }
-    }
-    switch (videoParam.mfx.FrameInfo.PicStruct)
-    {
-        case MFX_PICSTRUCT_PROGRESSIVE: // default
-            break;
-        case MFX_PICSTRUCT_FIELD_TFF:
-            hb_log("encqsvInit: PicStruct top field first");
-            break;
-        case MFX_PICSTRUCT_FIELD_BFF:
-            hb_log("encqsvInit: PicStruct bottom field first");
-            break;
-        default:
-            hb_error("encqsvInit: invalid PicStruct value 0x%"PRIx16"",
-                     videoParam.mfx.FrameInfo.PicStruct);
-            return -1;
-    }
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_OPTION1)
-    {
-        if (videoParam.mfx.CodecId == MFX_CODEC_AVC)
-        {
-            if (option1->CAVLC != MFX_CODINGOPTION_OFF)
-            {
-                hb_log("encqsvInit: CAVLC %s",
-                       hb_qsv_codingoption_get_name(option1->CAVLC));
-            }
-        }
-    }
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_OPTION2_EXTBRC)
-    {
-        if (option2->ExtBRC != MFX_CODINGOPTION_OFF)
-        {
-            hb_log("encqsvInit: ExtBRC %s",
-                   hb_qsv_codingoption_get_name(option2->ExtBRC));
-        }
-    }
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_OPTION2_MBBRC)
-    {
-        if (option2->MBBRC != MFX_CODINGOPTION_ON)
-        {
-            hb_log("encqsvInit: MBBRC %s",
-                   hb_qsv_codingoption_get_name(option2->MBBRC));
-        }
-    }
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_OPTION2_TRELLIS)
-    {
-        switch (option2->Trellis)
-        {
-            case MFX_TRELLIS_OFF: // default
-                break;
-            case MFX_TRELLIS_UNKNOWN:
-                hb_log("encqsvInit: Trellis unknown (auto)");
-                break;
-            default:
-                hb_log("encqsvInit: Trellis on (%s%s%s)",
-                       (option2->Trellis & MFX_TRELLIS_I) ? "I" : "",
-                       (option2->Trellis & MFX_TRELLIS_P) &&
-                       (videoParam.mfx.GopPicSize > 1)    ? "P" : "",
-                       (option2->Trellis & MFX_TRELLIS_B) &&
-                       (videoParam.mfx.GopRefDist > 1)    ? "B" : "");
-                break;
-        }
-    }
-
     // AsyncDepth has now been set and/or modified by Media SDK
     // fall back to default if zero
     pv->max_async_depth = videoParam.AsyncDepth ? videoParam.AsyncDepth : HB_QSV_ASYNC_DEPTH_DEFAULT;
@@ -1764,8 +1817,7 @@ void encqsvClose(hb_work_object_t *w)
                 hb_qsv_unload_plugins(&pv->loaded_plugins, qsv_ctx->mfx_session, version);
             }
 
-            /* QSV context cleanup and MFXClose */
-            hb_qsv_context_clean(qsv_ctx);
+            hb_qsv_uninit_enc(pv->job);
 
             hb_display_close(&pv->display);
 
@@ -1809,14 +1861,7 @@ void encqsvClose(hb_work_object_t *w)
                 }
                 qsv_enc_space->is_init_done = 0;
             }
-
-            if (pv->is_sys_mem)
-            {
-                av_freep(&qsv_ctx);
-            }
         }
-
-        hb_qsv_uninit_enc();
     }
 
     if (pv != NULL)
@@ -2056,8 +2101,10 @@ fail:
 
 static int qsv_enc_work(hb_work_private_t *pv,
                         hb_qsv_list *qsv_atom,
-                        mfxFrameSurface1 *surface)
+                        mfxFrameSurface1 *surface,
+                        HBQSVFramesContext *frames_ctx)
 {
+    int err;
     mfxStatus sts;
     hb_qsv_context *qsv_ctx       = pv->job->qsv.ctx;
     hb_qsv_space   *qsv_enc_space = pv->job->qsv.ctx->enc_space;
@@ -2083,7 +2130,7 @@ static int qsv_enc_work(hb_work_private_t *pv,
             {
                 if(!pv->is_sys_mem && surface)
                 {
-                    hb_qsv_release_surface_from_pool(surface->Data.MemId);
+                    hb_qsv_release_surface_from_pool_by_surface_pointer(frames_ctx, surface);
                 }
 
                 if (qsv_atom != NULL)
@@ -2105,14 +2152,20 @@ static int qsv_enc_work(hb_work_private_t *pv,
             }
             else
             {
-                hb_qsv_stage *new_stage = hb_qsv_stage_init();
-                new_stage->type         = HB_QSV_ENCODE;
-                new_stage->in.p_surface = surface;
-                new_stage->out.sync     = qsv_enc_space->p_syncp[sync_idx];
-                new_stage->out.p_bs     = task->bs;
-                task->stage             = new_stage;
+                hb_qsv_stage *new_stage        = hb_qsv_stage_init();
+                new_stage->type                = HB_QSV_ENCODE;
+                new_stage->in.p_surface        = surface;
+                new_stage->in.p_frames_ctx     = frames_ctx;
+                new_stage->out.sync            = qsv_enc_space->p_syncp[sync_idx];
+                new_stage->out.p_bs            = task->bs;
+                task->stage                    = new_stage;
                 pv->async_depth++;
 
+                if(!pv->is_sys_mem && surface)
+                {
+                    hb_qsv_release_surface_from_pool_by_surface_pointer(frames_ctx, surface);
+                }
+
                 if (qsv_atom != NULL)
                 {
                     hb_qsv_add_stagee(&qsv_atom, new_stage, HAVE_THREADS);
@@ -2155,12 +2208,11 @@ static int qsv_enc_work(hb_work_private_t *pv,
                 pv->async_depth--;
 
                 /* perform a sync operation to get the output bitstream */
-                hb_qsv_wait_on_sync(qsv_ctx, task->stage);
-
-                mfxFrameSurface1 *surface = task->stage->in.p_surface;
-                if(!pv->is_sys_mem && surface)
+                err = hb_qsv_wait_on_sync(qsv_ctx, task->stage);
+                if (err < 0)
                 {
-                    hb_qsv_release_surface_from_pool(surface->Data.MemId);
+                    hb_error("encqsv: hb_qsv_wait_on_sync failed (%d)", err);
+                    return err;
                 }
 
                 if (task->bs->DataLength > 0)
@@ -2191,6 +2243,7 @@ static int qsv_enc_work(hb_work_private_t *pv,
 
 int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
 {
+    int err;
     hb_work_private_t *pv = w->private_data;
     hb_buffer_t *in       = *buf_in;
     hb_job_t *job         = pv->job;
@@ -2210,14 +2263,20 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
      */
     if (in->s.flags & HB_BUF_FLAG_EOF)
     {
-        qsv_enc_work(pv, NULL, NULL);
+        err = qsv_enc_work(pv, NULL, NULL, NULL);
+        if (err < 0)
+        {
+            hb_error("encqsvWork: EOF qsv_enc_work failed %d", err);
+            goto fail;
+        }
         hb_buffer_list_append(&pv->encoded_frames, in);
         *buf_out = hb_buffer_list_clear(&pv->encoded_frames);
         *buf_in = NULL; // don't let 'work_loop' close this buffer
         return HB_WORK_DONE;
     }
 
-    mfxFrameSurface1 *surface       = NULL;
+    mfxFrameSurface1   *surface     = NULL;
+    HBQSVFramesContext *frames_ctx  = NULL;
     hb_qsv_list      *qsv_atom      = NULL;
     hb_qsv_context   *qsv_ctx       = job->qsv.ctx;
     hb_qsv_space     *qsv_enc_space = job->qsv.ctx->enc_space;
@@ -2240,23 +2299,16 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
     {
 #if HB_PROJECT_FEATURE_QSV
         QSVMid *mid = NULL;
-        if(in->qsv_details.frame)
+        if (in->qsv_details.frame && in->qsv_details.frame->data[3])
         {
             surface = ((mfxFrameSurface1*)in->qsv_details.frame->data[3]);
-            mid = surface->Data.MemId;
-        }
-        else
-        {
-            // Create black buffer in the begining of the encoding, usually first 2 frames
-            hb_qsv_get_free_surface_from_pool(HB_POOL_SURFACE_SIZE - HB_POOL_ENCODER_SIZE, HB_POOL_SURFACE_SIZE, &mid, &surface);
-        }
-
-        if(surface)
-        {
-            hb_qsv_replace_surface_mid(mid, surface);
+            frames_ctx = in->qsv_details.qsv_frames_ctx;
+            hb_qsv_get_mid_by_surface_from_pool(frames_ctx, surface, &mid);
+            hb_qsv_replace_surface_mid(frames_ctx, mid, surface);
         }
         else
         {
+            hb_error("encqsv: in->qsv_details no surface available");
             goto fail;
         }
 #endif
@@ -2307,15 +2359,17 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
      *   the encoder to make sure the next frame is a keyframe
      *
      * The hard reset ensures encoding resumes with a clean state, avoiding
-     * miscellaneous hard-to-disagnose issues that may occur when resuming
+     * miscellaneous hard-to-diagnose issues that may occur when resuming
      * an encode after flushing the encoder or using MFXVideoENCODE_Reset.
      */
     if (in->s.new_chap > 0 && job->chapter_markers)
     {
         mfxStatus sts;
 
-        if (qsv_enc_work(pv, NULL, NULL) < 0)
+        err = qsv_enc_work(pv, NULL, NULL, NULL);
+        if (err < 0)
         {
+            hb_error("encqsvWork: new_chap qsv_enc_work failed %d", err);
             goto fail;
         }
 
@@ -2352,8 +2406,10 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
     /*
      * Now that the input surface is setup, we can encode it.
      */
-    if (qsv_enc_work(pv, qsv_atom, surface) < 0)
+    err = qsv_enc_work(pv, qsv_atom, surface, frames_ctx);
+    if (err < 0)
     {
+        hb_error("encqsvWork: qsv_enc_work failed %d", err);
         goto fail;
     }
 #if HB_PROJECT_FEATURE_QSV
