@@ -51,9 +51,6 @@ struct hb_work_private_s
     } frame_info[FRAME_INFO_SIZE];
 
     hb_chapter_queue_t * chapter_queue;
-
-    struct SwsContext  * sws;
-    hb_buffer_t        * sws_buf;
 };
 
 int  encavcodecInit( hb_work_object_t *, hb_job_t * );
@@ -117,6 +114,26 @@ static const char * const h265_mf_profile_name[] =
     "auto", "main",  NULL
 };
 
+static const enum AVPixelFormat pix_fmts[] =
+{
+    AV_PIX_FMT_YUV420P, AV_PIX_FMT_NONE
+};
+
+static const enum AVPixelFormat h26x_vt_pix_fmts[] =
+{
+    AV_PIX_FMT_NV12, AV_PIX_FMT_YUV420P, AV_PIX_FMT_NONE
+};
+
+static const enum AVPixelFormat h265_10bit_vt_pix_fmts[] =
+{
+    AV_PIX_FMT_P010LE, AV_PIX_FMT_NONE
+};
+
+static const enum AVPixelFormat h26x_mf_pix_fmts[] =
+{
+    AV_PIX_FMT_NV12, AV_PIX_FMT_NONE
+};
+
 int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
 {
     int ret = 0;
@@ -842,7 +859,7 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
     }
     context->width     = job->width;
     context->height    = job->height;
-    context->pix_fmt   = job->pix_fmt;
+    context->pix_fmt   = job->output_pix_fmt;
 
     context->sample_aspect_ratio.num = job->par.num;
     context->sample_aspect_ratio.den = job->par.den;
@@ -911,22 +928,6 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
     {
         av_dict_set(&av_opts, "profile", "main10", 0);
         context->max_b_frames = 16;
-
-        if (job->pix_fmt != AV_PIX_FMT_P010LE)
-        {
-            pv->sws = hb_sws_get_context(
-                                         job->width, job->height,
-                                         context->pix_fmt, job->color_range,
-                                         job->width, job->height,
-                                         AV_PIX_FMT_P010LE, job->color_range,
-                                         SWS_LANCZOS|SWS_ACCURATE_RND,
-                                         SWS_CS_DEFAULT);
-
-            pv->sws_buf = hb_frame_buffer_init(
-                             AV_PIX_FMT_P010LE, job->width, job->height);
-
-            context->pix_fmt = AV_PIX_FMT_P010LE;
-        }
     }
 
     if (job->vcodec == HB_VCODEC_FFMPEG_VCE_H264)
@@ -1033,22 +1034,6 @@ int encavcodecInit( hb_work_object_t * w, hb_job_t * job )
         job->vcodec == HB_VCODEC_FFMPEG_MF_H265)
     {
         av_dict_set(&av_opts, "hw_encoding", "1", 0);
-
-        if (job->pix_fmt != AV_PIX_FMT_NV12)
-        {
-            pv->sws = hb_sws_get_context(
-                                         job->width, job->height,
-                                         context->pix_fmt, job->color_range,
-                                         job->width, job->height,
-                                         AV_PIX_FMT_NV12, job->color_range,
-                                         SWS_LANCZOS|SWS_ACCURATE_RND,
-                                         SWS_CS_DEFAULT);
-
-            pv->sws_buf = hb_frame_buffer_init(
-                             AV_PIX_FMT_NV12, job->width, job->height);
-
-            context->pix_fmt = AV_PIX_FMT_NV12;
-        }
     }
 
     if (job->vcodec == HB_VCODEC_FFMPEG_MF_H265)
@@ -1207,14 +1192,6 @@ void encavcodecClose( hb_work_object_t * w )
         }
         hb_avcodec_free_context(&pv->context);
     }
-    if (pv->sws != NULL)
-    {
-        sws_freeContext(pv->sws);
-    }
-    if (pv->sws_buf != NULL)
-    {
-        hb_buffer_close(&pv->sws_buf);
-    }
     if( pv->file )
     {
         fclose( pv->file );
@@ -1374,25 +1351,6 @@ static void Encode( hb_work_object_t *w, hb_buffer_t *in,
     frame.linesize[1] = in->plane[1].stride;
     frame.linesize[2] = in->plane[2].stride;
 
-    if (pv->sws)
-    {
-        uint8_t *srcs[]   = { in->plane[0].data, in->plane[1].data, in->plane[2].data };
-        int srcs_stride[] = { in->plane[0].stride, in->plane[1].stride, in->plane[2].stride };
-        uint8_t *dsts[]   = { pv->sws_buf->plane[0].data, pv->sws_buf->plane[1].data, NULL };
-        int dsts_stride[] = { pv->sws_buf->plane[0].stride, pv->sws_buf->plane[1].stride, 0 };
-
-        sws_scale(pv->sws,
-                  (const uint8_t* const*)srcs, srcs_stride,
-                  0, in->f.height, dsts, dsts_stride);
-
-        for (int i = 0; i < 3; i++)
-        {
-            frame.data[i] = dsts[i];
-            frame.linesize[i] = dsts_stride[i];
-        }
-        frame.format = pv->context->pix_fmt;
-    }
-
     if (in->s.new_chap > 0 && pv->job->chapter_markers)
     {
         /* chapters have to start with an IDR frame so request that this
@@ -1634,3 +1592,21 @@ const char* const* hb_av_profile_get_names(int encoder)
              return NULL;
      }
 }
+
+const int* hb_av_get_pix_fmts(int encoder)
+{
+    switch (encoder)
+    {
+        case HB_VCODEC_FFMPEG_VT_H264:
+        case HB_VCODEC_FFMPEG_VT_H265:
+            return h26x_vt_pix_fmts;
+        case HB_VCODEC_FFMPEG_VT_H265_10BIT:
+            return h265_10bit_vt_pix_fmts;
+        case HB_VCODEC_FFMPEG_MF_H264:
+        case HB_VCODEC_FFMPEG_MF_H265:
+            return h26x_mf_pix_fmts;
+
+         default:
+             return pix_fmts;
+     }
+}
