@@ -1,6 +1,6 @@
 /* common.h
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -139,14 +139,10 @@ void      * hb_list_item( const hb_list_t *, int );
 void        hb_list_close( hb_list_t ** );
 
 void hb_reduce( int *x, int *y, int num, int den );
-void hb_limit_rational( int *x, int *y, int num, int den, int limit );
+void hb_limit_rational( int *x, int *y, int64_t num, int64_t den, int limit );
 void hb_reduce64( int64_t *x, int64_t *y, int64_t num, int64_t den );
 void hb_limit_rational64( int64_t *x, int64_t *y, int64_t num, int64_t den, int64_t limit );
 
-#define HB_KEEP_WIDTH           0x01
-#define HB_KEEP_HEIGHT          0x02
-#define HB_KEEP_DISPLAY_ASPECT  0x04
-
 void hb_job_set_encoder_preset (hb_job_t *job, const char *preset);
 void hb_job_set_encoder_tune   (hb_job_t *job, const char *tune);
 void hb_job_set_encoder_options(hb_job_t *job, const char *options);
@@ -188,16 +184,8 @@ void hb_attachment_close(hb_attachment_t **attachment);
 hb_metadata_t * hb_metadata_init(void);
 hb_metadata_t * hb_metadata_copy(const hb_metadata_t *src);
 void hb_metadata_close(hb_metadata_t **metadata);
-void hb_metadata_set_name( hb_metadata_t *metadata, const char *name );
-void hb_metadata_set_artist( hb_metadata_t *metadata, const char *artist );
-void hb_metadata_set_composer( hb_metadata_t *metadata, const char *composer );
-void hb_metadata_set_release_date( hb_metadata_t *metadata, const char *release_date );
-void hb_metadata_set_comment( hb_metadata_t *metadata, const char *comment );
-void hb_metadata_set_genre( hb_metadata_t *metadata, const char *genre );
-void hb_metadata_set_album( hb_metadata_t *metadata, const char *album );
-void hb_metadata_set_album_artist( hb_metadata_t *metadata, const char *album_artist );
-void hb_metadata_set_description( hb_metadata_t *metadata, const char *description );
-void hb_metadata_set_long_description( hb_metadata_t *metadata, const char *long_description );
+void hb_update_meta_dict(hb_dict_t * dict, const char * key, const char * value);
+const char * hb_lookup_meta_key(const char * mux_key);
 void hb_metadata_add_coverart( hb_metadata_t *metadata, const uint8_t *data, int size, int type );
 void hb_metadata_rem_coverart( hb_metadata_t *metadata, int ii );
 
@@ -254,6 +242,16 @@ struct hb_rational_s
     int den;
 };
 
+static inline hb_rational_t hb_make_q(int num, int den)
+{
+    hb_rational_t r = { num, den };
+    return r;
+}
+
+static inline double hb_q2d(hb_rational_t a){
+    return a.num / (double) a.den;
+}
+
 struct hb_geometry_s
 {
     int width;
@@ -261,15 +259,38 @@ struct hb_geometry_s
     hb_rational_t par;
 };
 
+struct hb_geometry_crop_s
+{
+    int crop[4];
+    int pad[4];
+    hb_geometry_t geometry;
+};
+
+// hb_geometry_settings_s 'keep' flags
+#define HB_KEEP_WIDTH           0x01
+#define HB_KEEP_HEIGHT          0x02
+#define HB_KEEP_DISPLAY_ASPECT  0x04
+#define HB_KEEP_DISPLAY_WIDTH   0x08
+#define HB_KEEP_PAD             0x10
+
+// hb_geometry_settings_s 'flags'
+#define HB_GEO_SCALE_UP         0x01
+#define HB_GEO_SCALE_BEST       0x02
+
 struct hb_geometry_settings_s
 {
-    int mode;                   // Anamorphic mode, see job struct anamorphic
-    int keep;                   // Specifies settings that shouldn't be changed
-    int itu_par;                // use dvd dimensions to determine PAR
-    int modulus;                // pixel alignment for loose anamorphic
-    int crop[4];                // Pixels cropped from source before scaling
-    int maxWidth;               // max destination storage width
-    int maxHeight;              // max destination storage height
+    int mode;               // Anamorphic mode, see job struct anamorphic
+    int keep;               // Specifies settings that shouldn't be changed
+    int flags;              // Flags that affect calculations
+    int itu_par;            // use dvd dimensions to determine PAR
+    int modulus;            // pixel alignment for loose anamorphic
+    int crop[4];            // Pixels cropped from source before scaling
+    int pad[4];             // Pixels cropped from source before scaling
+    int maxWidth;           // max destination storage width
+    int maxHeight;          // max destination storage height
+    int displayWidth;       // display width, used with !HB_KEEP_DISPLAY_ASPECT
+    int displayHeight;      // display height, used with !HB_KEEP_DISPLAY_ASPECT
+                            // Note that display size includes pad!
     hb_geometry_t geometry;
 };
 
@@ -280,6 +301,9 @@ struct hb_image_s
     int max_plane;
     int width;
     int height;
+    int color_prim;
+    int color_transfer;
+    int color_matrix;
     uint8_t *data;
 
     struct image_plane
@@ -309,6 +333,20 @@ struct hb_subtitle_config_s
     int64_t      offset;
 };
 
+struct hb_mastering_display_metadata_s {
+    hb_rational_t display_primaries[3][2];
+    hb_rational_t white_point[2];
+    hb_rational_t min_luminance;
+    hb_rational_t max_luminance;
+    int has_primaries;
+    int has_luminance;
+};
+
+struct hb_content_light_metadata_s {
+    unsigned max_cll;
+    unsigned max_fall;
+};
+
 /*******************************************************************************
  * Lists of rates, mixdowns, encoders etc.
  *******************************************************************************
@@ -368,12 +406,16 @@ const hb_rate_t* hb_audio_bitrate_get_next(const hb_rate_t *last);
 
 void        hb_video_quality_get_limits(uint32_t codec, float *low, float *high, float *granularity, int *direction);
 const char* hb_video_quality_get_name(uint32_t codec);
+int         hb_video_quality_is_supported(uint32_t codec);
 
+int                hb_video_encoder_is_supported(int encoder);
+int                hb_video_encoder_pix_fmt_is_supported(int encoder, int pix_fmt);
 int                hb_video_encoder_get_depth   (int encoder);
 const char* const* hb_video_encoder_get_presets (int encoder);
 const char* const* hb_video_encoder_get_tunes   (int encoder);
 const char* const* hb_video_encoder_get_profiles(int encoder);
 const char* const* hb_video_encoder_get_levels  (int encoder);
+const int*         hb_video_encoder_get_pix_fmts(int encoder);
 
 void  hb_audio_quality_get_limits(uint32_t codec, float *low, float *high, float *granularity, int *direction);
 float hb_audio_quality_get_best(uint32_t codec, float quality);
@@ -385,7 +427,7 @@ float hb_audio_compression_get_default(uint32_t codec);
 
 int                hb_audio_dither_get_default(void);
 int                hb_audio_dither_get_default_method(void); // default method, if enabled && supported
-int                hb_audio_dither_is_supported(uint32_t codec);
+int                hb_audio_dither_is_supported(uint32_t codec, int depth);
 int                hb_audio_dither_get_from_name(const char *name);
 const char*        hb_audio_dither_get_description(int method);
 const hb_dither_t* hb_audio_dither_get_next(const hb_dither_t *last);
@@ -510,7 +552,7 @@ struct hb_job_s
          cfr:               0 (vfr), 1 (cfr), 2 (pfr) [see render.c]
          pass:              0, 1 or 2 (or -1 for scan)
          areBframes:        boolean to note if b-frames are used */
-#define HB_VCODEC_MASK         0x0FFFFFF
+#define HB_VCODEC_MASK         0x7FFFFFF
 #define HB_VCODEC_INVALID      0x0000000
 #define HB_VCODEC_THEORA       0x0000002
 #define HB_VCODEC_FFMPEG_MPEG4 0x0000010
@@ -523,7 +565,10 @@ struct hb_job_s
 #define HB_VCODEC_FFMPEG_NVENC_H265 0x00200000
 #define HB_VCODEC_FFMPEG_VT_H264 0x00400000
 #define HB_VCODEC_FFMPEG_VT_H265 0x00800000
-#define HB_VCODEC_FFMPEG_MASK  (0x00000F0|HB_VCODEC_FFMPEG_VCE_H264|HB_VCODEC_FFMPEG_VCE_H265|HB_VCODEC_FFMPEG_NVENC_H264|HB_VCODEC_FFMPEG_NVENC_H265|HB_VCODEC_FFMPEG_VT_H264|HB_VCODEC_FFMPEG_VT_H265)
+#define HB_VCODEC_FFMPEG_VT_H265_10BIT 0x01000000
+#define HB_VCODEC_FFMPEG_MF_H264 0x02000000
+#define HB_VCODEC_FFMPEG_MF_H265 0x04000000
+#define HB_VCODEC_FFMPEG_MASK  (0x00000F0|HB_VCODEC_FFMPEG_VCE_H264|HB_VCODEC_FFMPEG_VCE_H265|HB_VCODEC_FFMPEG_NVENC_H264|HB_VCODEC_FFMPEG_NVENC_H265|HB_VCODEC_FFMPEG_VT_H264|HB_VCODEC_FFMPEG_VT_H265|HB_VCODEC_FFMPEG_VT_H265_10BIT|HB_VCODEC_FFMPEG_MF_H264|HB_VCODEC_FFMPEG_MF_H265)
 #define HB_VCODEC_QSV_H264     0x0000100
 #define HB_VCODEC_QSV_H265_8BIT     0x0000200
 #define HB_VCODEC_QSV_H265_10BIT    0x0000400
@@ -534,14 +579,14 @@ struct hb_job_s
 #define HB_VCODEC_X264         HB_VCODEC_X264_8BIT
 #define HB_VCODEC_X264_10BIT   0x0020000
 #define HB_VCODEC_X264_MASK    0x0030000
-#define HB_VCODEC_H264_MASK    (HB_VCODEC_X264_MASK|HB_VCODEC_QSV_H264|HB_VCODEC_FFMPEG_VCE_H264|HB_VCODEC_FFMPEG_NVENC_H264|HB_VCODEC_FFMPEG_VT_H264)
+#define HB_VCODEC_H264_MASK    (HB_VCODEC_X264_MASK|HB_VCODEC_QSV_H264|HB_VCODEC_FFMPEG_VCE_H264|HB_VCODEC_FFMPEG_NVENC_H264|HB_VCODEC_FFMPEG_VT_H264|HB_VCODEC_FFMPEG_MF_H264)
 #define HB_VCODEC_X265_8BIT    0x0001000
 #define HB_VCODEC_X265         HB_VCODEC_X265_8BIT
 #define HB_VCODEC_X265_10BIT   0x0002000
 #define HB_VCODEC_X265_12BIT   0x0004000
 #define HB_VCODEC_X265_16BIT   0x0008000
 #define HB_VCODEC_X265_MASK    0x000F000
-#define HB_VCODEC_H265_MASK    (HB_VCODEC_X265_MASK|HB_VCODEC_QSV_H265_MASK|HB_VCODEC_FFMPEG_VCE_H265|HB_VCODEC_FFMPEG_NVENC_H265|HB_VCODEC_FFMPEG_VT_H265)
+#define HB_VCODEC_H265_MASK    (HB_VCODEC_X265_MASK|HB_VCODEC_QSV_H265_MASK|HB_VCODEC_FFMPEG_VCE_H265|HB_VCODEC_FFMPEG_NVENC_H265|HB_VCODEC_FFMPEG_VT_H265|HB_VCODEC_FFMPEG_VT_H265_10BIT|HB_VCODEC_FFMPEG_MF_H265)
 
 /* define an invalid CQ value compatible with all CQ-capable codecs */
 #define HB_INVALID_VIDEO_QUALITY (-1000.)
@@ -565,7 +610,10 @@ struct hb_job_s
     char           *encoder_level;
     int             areBframes;
 
-    int             pix_fmt;
+    // Pixel format from decoder to the end of the filters chain
+    int             input_pix_fmt;
+    // Pixel format from the end of filters chain to the encoder
+    int             output_pix_fmt;
     int             color_prim;
     int             color_transfer;
     int             color_matrix;
@@ -624,6 +672,9 @@ struct hb_job_s
 #define HB_COLR_MAT_ICTCP        14 // ITU-R BT.2100-0, ICtCp
 // 0, 3-5, 8, 11-65535: reserved/not implemented
 
+    hb_mastering_display_metadata_t mastering;
+    hb_content_light_metadata_t coll;
+
     hb_list_t     * list_chapter;
 
     /* List of audio settings. */
@@ -760,6 +811,7 @@ struct hb_job_s
 #define HB_ACODEC_FFAAC     0x00010000
 #define HB_ACODEC_FFMPEG    0x00020000
 #define HB_ACODEC_DCA_HD    0x00040000
+#define HB_ACODEC_MP2       0x06000000
 #define HB_ACODEC_MP3       0x00080000
 #define HB_ACODEC_FFFLAC    0x00100000
 #define HB_ACODEC_FFFLAC24  0x00200000
@@ -770,7 +822,7 @@ struct hb_job_s
 #define HB_ACODEC_OPUS      0x04000000
 #define HB_ACODEC_FF_MASK   0x07FF2800
 #define HB_ACODEC_PASS_FLAG 0x40000000
-#define HB_ACODEC_PASS_MASK   (HB_ACODEC_AC3 | HB_ACODEC_DCA | HB_ACODEC_DCA_HD | HB_ACODEC_FFAAC | HB_ACODEC_FFEAC3 | HB_ACODEC_FFFLAC | HB_ACODEC_MP3 | HB_ACODEC_FFTRUEHD)
+#define HB_ACODEC_PASS_MASK   (HB_ACODEC_AC3 | HB_ACODEC_DCA | HB_ACODEC_DCA_HD | HB_ACODEC_FFAAC | HB_ACODEC_FFEAC3 | HB_ACODEC_FFFLAC | HB_ACODEC_MP2 | HB_ACODEC_MP3 | HB_ACODEC_FFTRUEHD)
 #define HB_ACODEC_AUTO_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_PASS_MASK)
 #define HB_ACODEC_ANY         (HB_ACODEC_PASS_FLAG | HB_ACODEC_MASK)
 #define HB_ACODEC_AAC_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFAAC)
@@ -779,6 +831,7 @@ struct hb_job_s
 #define HB_ACODEC_DCA_HD_PASS (HB_ACODEC_PASS_FLAG | HB_ACODEC_DCA_HD)
 #define HB_ACODEC_EAC3_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFEAC3)
 #define HB_ACODEC_FLAC_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFFLAC)
+#define HB_ACODEC_MP2_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_MP2)
 #define HB_ACODEC_MP3_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_MP3)
 #define HB_ACODEC_TRUEHD_PASS (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFTRUEHD)
 
@@ -846,7 +899,7 @@ struct hb_audio_config_s
         PRIVATE uint32_t reg_desc; /* Registration descriptor of source */
         PRIVATE uint32_t stream_type; /* Stream type from source stream */
         PRIVATE uint32_t substream_type; /* Substream type for multiplexed streams */
-        PRIVATE uint32_t version; /* Bitsream version */
+        PRIVATE uint32_t version; /* Bitstream version */
         PRIVATE uint32_t flags; /* Bitstream flags, codec-specific */
         PRIVATE uint32_t mode; /* Bitstream mode, codec-specific */
         PRIVATE int samplerate; /* Input sample rate (Hz) */
@@ -966,13 +1019,23 @@ struct hb_subtitle_s
     hb_subtitle_config_t config;
 
     enum subtype { PICTURESUB, TEXTSUB } format;
-    enum subsource { VOBSUB, CC608SUB, /*unused*/CC708SUB,
-                     UTF8SUB, TX3GSUB, SSASUB, PGSSUB,
-                     IMPORTSRT, IMPORTSSA, SRTSUB = IMPORTSRT } source;
+    enum subsource {
+        VOBSUB,
+        CC608SUB,
+        CC708SUB, // unused
+        UTF8SUB,
+        TX3GSUB,
+        SSASUB,
+        PGSSUB,
+        IMPORTSRT,
+        IMPORTSSA,
+        DVBSUB,
+        SRTSUB = IMPORTSRT
+    } source;
     const char * name;
     char         lang[1024];
     char         iso639_2[4];
-    uint32_t     attributes; /* Closed Caption, Childrens, Directors etc */
+    uint32_t     attributes; /* Closed Caption, Children, Directors etc */
 
     // Color lookup table for VOB subtitle tracks. Each entry is in YCbCr format.
     // Must be filled out by the demuxer for VOB subtitle tracks.
@@ -991,6 +1054,7 @@ struct hb_subtitle_s
 #ifdef __LIBHB__
     /* Internal data */
     uint32_t        codec;          /* Input "codec" */
+    uint32_t        codec_param;    /* Per-codec config info */
     uint32_t        reg_desc;       /* registration descriptor of source */
     uint32_t        stream_type;    /* stream type from source stream */
     uint32_t        substream_type; /* substream for multiplexed streams */
@@ -1031,16 +1095,7 @@ struct hb_coverart_s
 
 struct hb_metadata_s
 {
-    char  *name;
-    char  *artist;          // Actors
-    char  *composer;
-    char  *release_date;
-    char  *comment;
-    char  *album;           // DVD
-    char  *album_artist;    // Director
-    char  *genre;
-    char  *description;
-    char  *long_description;
+    hb_dict_t * dict;
     hb_list_t * list_coverart;
 };
 
@@ -1075,6 +1130,8 @@ struct hb_title_s
     int             color_transfer;
     int             color_matrix;
     int             color_range;
+    hb_mastering_display_metadata_t mastering;
+    hb_content_light_metadata_t     coll;
     hb_rational_t   vrate;
     int             crop[4];
     enum {HB_DVD_DEMUXER, HB_TS_DEMUXER, HB_PS_DEMUXER, HB_NULL_DEMUXER} demuxer;
@@ -1251,13 +1308,11 @@ extern hb_work_object_t hb_sync_video;
 extern hb_work_object_t hb_sync_audio;
 extern hb_work_object_t hb_sync_subtitle;
 extern hb_work_object_t hb_decvobsub;
-extern hb_work_object_t hb_encvobsub;
-extern hb_work_object_t hb_deccc608;
 extern hb_work_object_t hb_decsrtsub;
 extern hb_work_object_t hb_decutf8sub;
 extern hb_work_object_t hb_dectx3gsub;
 extern hb_work_object_t hb_decssasub;
-extern hb_work_object_t hb_decpgssub;
+extern hb_work_object_t hb_decavsub;
 extern hb_work_object_t hb_encavcodec;
 extern hb_work_object_t hb_encqsv;
 extern hb_work_object_t hb_encx264;
@@ -1363,14 +1418,15 @@ enum
     HB_FILTER_HQDN3D = HB_FILTER_DENOISE,
     HB_FILTER_NLMEANS,
     HB_FILTER_CHROMA_SMOOTH,
+    HB_FILTER_ROTATE,
     HB_FILTER_RENDER_SUB,
     HB_FILTER_CROP_SCALE,
     HB_FILTER_LAPSHARP,
     HB_FILTER_UNSHARP,
-    HB_FILTER_ROTATE,
     HB_FILTER_GRAYSCALE,
     HB_FILTER_PAD,
     HB_FILTER_COLORSPACE,
+    HB_FILTER_FORMAT,
 
     // Finally filters that don't care what order they are in,
     // except that they must be after the above filters
@@ -1389,6 +1445,8 @@ hb_filter_object_t * hb_filter_get( int filter_id );
 hb_filter_object_t * hb_filter_init( int filter_id );
 hb_filter_object_t * hb_filter_copy( hb_filter_object_t * filter );
 hb_list_t          * hb_filter_list_copy(const hb_list_t *src);
+hb_dict_t          * hb_filter_dict_find(const hb_value_array_t * list,
+                                         int filter_id);
 hb_filter_object_t * hb_filter_find(const hb_list_t *list, int filter_id);
 void                 hb_filter_close( hb_filter_object_t ** );
 void                 hb_filter_info_close( hb_filter_info_t ** );
@@ -1416,6 +1474,7 @@ char ** hb_str_vsplit( const char * str, char delem );
 
 int hb_yuv2rgb(int yuv);
 int hb_rgb2yuv(int rgb);
+int hb_rgb2yuv_bt709(int rgb);
 
 const char * hb_subsource_name( int source );
 
@@ -1437,6 +1496,9 @@ int hb_output_color_prim(hb_job_t * job);
 int hb_output_color_transfer(hb_job_t * job);
 int hb_output_color_matrix(hb_job_t * job);
 
+int hb_get_bit_depth(int format);
+int hb_get_best_pix_fmt(hb_job_t * job);
+
 #define HB_NEG_FLOAT_REG "(([-])?(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
 #define HB_FLOAT_REG     "(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
 #define HB_NEG_INT_REG   "(([-]?[0-9]+)"
