@@ -1,6 +1,6 @@
 /* json.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -117,7 +117,7 @@ hb_dict_t* hb_state_to_dict( hb_state_t * state)
     }
     if (dict == NULL)
     {
-        hb_error("json pack failure: %s", error.text);
+        hb_error("hb_state_to_dict, json pack failure: %s", error.text);
     }
     return dict;
 }
@@ -143,7 +143,7 @@ hb_dict_t * hb_version_dict()
         "Arch",          hb_value_string(HB_PROJECT_HOST_ARCH));
     if (dict == NULL)
     {
-        hb_error("json pack failure: %s", error.text);
+        hb_error("hb_version_dict, json pack failure: %s", error.text);
         return NULL;
     }
 
@@ -185,7 +185,7 @@ hb_dict_t * hb_audio_attributes_to_dict(uint32_t attributes)
         "Default",          hb_value_bool(attributes & HB_AUDIO_ATTR_DEFAULT));
     if (dict == NULL)
     {
-        hb_error("json pack failure: %s", error.text);
+        hb_error("hb_audio_attributes_to_dict, json pack failure: %s", error.text);
     }
     return dict;
 }
@@ -211,7 +211,7 @@ hb_dict_t * hb_subtitle_attributes_to_dict(uint32_t attributes)
         "Default",       hb_value_bool(attributes & HB_SUBTITLE_ATTR_DEFAULT));
     if (dict == NULL)
     {
-        hb_error("json pack failure: %s", error.text);
+        hb_error("hb_subtitle_attributes_to_dict, json pack failure: %s", error.text);
     }
     return dict;
 }
@@ -242,7 +242,7 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         // InterlaceDetected, VideoCodec
         "s:o, s:o,"
         // Metadata
-        "s:{}"
+        "s:o"
     "}",
     "Type",                 hb_value_int(title->type),
     "Path",                 hb_value_string(title->path),
@@ -276,11 +276,11 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         "Den",              hb_value_int(title->vrate.den),
     "InterlaceDetected",    hb_value_bool(title->detected_interlacing),
     "VideoCodec",           hb_value_string(title->video_codec_name),
-    "Metadata"
+    "Metadata",             hb_value_dup(title->metadata->dict)
     );
     if (dict == NULL)
     {
-        hb_error("json pack failure: %s", error.text);
+        hb_error("hb_title_to_dict_internal, json pack failure: %s", error.text);
         return NULL;
     }
 
@@ -289,58 +289,6 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         hb_dict_set(dict, "Container", hb_value_string(title->container_name));
     }
 
-    // Add metadata
-    hb_dict_t *meta_dict = hb_dict_get(dict, "Metadata");
-    if (title->metadata->name != NULL)
-    {
-        hb_dict_set(meta_dict, "Name", hb_value_string(title->metadata->name));
-    }
-    if (title->metadata->artist != NULL)
-    {
-        hb_dict_set(meta_dict, "Artist",
-                    hb_value_string(title->metadata->artist));
-    }
-    if (title->metadata->composer != NULL)
-    {
-        hb_dict_set(meta_dict, "Composer",
-                    hb_value_string(title->metadata->composer));
-    }
-    if (title->metadata->comment != NULL)
-    {
-        hb_dict_set(meta_dict, "Comment",
-                    hb_value_string(title->metadata->comment));
-    }
-    if (title->metadata->genre != NULL)
-    {
-        hb_dict_set(meta_dict, "Genre",
-                    hb_value_string(title->metadata->genre));
-    }
-    if (title->metadata->album != NULL)
-    {
-        hb_dict_set(meta_dict, "Album",
-                    hb_value_string(title->metadata->album));
-    }
-    if (title->metadata->album_artist != NULL)
-    {
-        hb_dict_set(meta_dict, "AlbumArtist",
-                    hb_value_string(title->metadata->album_artist));
-    }
-    if (title->metadata->description != NULL)
-    {
-        hb_dict_set(meta_dict, "Description",
-                    hb_value_string(title->metadata->description));
-    }
-    if (title->metadata->long_description != NULL)
-    {
-        hb_dict_set(meta_dict, "LongDescription",
-                    hb_value_string(title->metadata->long_description));
-    }
-    if (title->metadata->release_date != NULL)
-    {
-        hb_dict_set(meta_dict, "ReleaseDate",
-                    hb_value_string(title->metadata->release_date));
-    }
-
     // process chapter list
     hb_dict_t * chapter_list = hb_value_array_init();
     for (ii = 0; ii < hb_list_count(title->list_chapter); ii++)
@@ -362,7 +310,7 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         );
         if (chapter_dict == NULL)
         {
-            hb_error("json pack failure: %s", error.text);
+            hb_error("hb_title_to_dict_internal, chapter, json pack failure: %s", error.text);
             return NULL;
         }
         hb_value_array_append(chapter_list, chapter_dict);
@@ -391,7 +339,8 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
 
         attributes = hb_audio_attributes_to_dict(audio->config.lang.attributes);
         audio_dict = json_pack_ex(&error, 0,
-        "{s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+        "{s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+            "TrackNumber",       hb_value_int(ii + 1),
             "Description",       hb_value_string(audio->config.lang.description),
             "Language",          hb_value_string(audio->config.lang.simple),
             "LanguageCode",      hb_value_string(audio->config.lang.iso639_2),
@@ -407,7 +356,7 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
             "LFECount",          hb_value_int(lfe_count));
         if (audio_dict == NULL)
         {
-            hb_error("json pack failure: %s", error.text);
+            hb_error("hb_title_to_dict_internal, audio, json pack failure: %s", error.text);
             return NULL;
         }
         if (audio->config.in.name != NULL)
@@ -429,7 +378,8 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
         format = subtitle->format == PICTURESUB ? "bitmap" : "text";
         attributes = hb_subtitle_attributes_to_dict(subtitle->attributes);
         subtitle_dict = json_pack_ex(&error, 0,
-            "{s:o, s:o, s:o, s:o, s:o, s:o}",
+            "{s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+            "TrackNumber",  hb_value_int(ii + 1),
             "Format",       hb_value_string(format),
             "Source",       hb_value_int(subtitle->source),
             "SourceName",   hb_value_string(hb_subsource_name(subtitle->source)),
@@ -438,7 +388,7 @@ static hb_dict_t* hb_title_to_dict_internal( hb_title_t *title )
             "LanguageCode", hb_value_string(subtitle->iso639_2));
         if (subtitle_dict == NULL)
         {
-            hb_error("json pack failure: %s", error.text);
+            hb_error("hb_title_to_dict_internal, subtitle, json pack failure: %s", error.text);
             return NULL;
         }
         if (subtitle->name != NULL)
@@ -528,6 +478,16 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     json_error_t error;
     int subtitle_search_burn;
     int ii;
+    int adapter_index;
+
+#if HB_PROJECT_FEATURE_QSV
+    if (job->qsv.ctx){
+        adapter_index = job->qsv.ctx->dx_index;
+    }
+    
+#else
+    adapter_index = 0;
+#endif
 
     if (job == NULL || job->title == NULL)
         return NULL;
@@ -536,7 +496,6 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     // necessary PAR value
 
     subtitle_search_burn = job->select_subtitle_config.dest == RENDERSUB;
-
     dict = json_pack_ex(&error, 0,
     "{"
     // SequenceID
@@ -548,14 +507,14 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     "s:{s:o, s:o, s:o,},"
     // PAR {Num, Den}
     "s:{s:o, s:o},"
-    // Video {Encoder, QSV {Decode, AsyncDepth}}
-    "s:{s:o, s:{s:o, s:o}},"
+    // Video {Encoder, QSV {Decode, AsyncDepth, AdapterIndex}}
+    "s:{s:o, s:{s:o, s:o, s:o}},"
     // Audio {CopyMask, FallbackEncoder, AudioList []}
     "s:{s:[], s:o, s:[]},"
     // Subtitles {Search {Enable, Forced, Default, Burn}, SubtitleList []}
     "s:{s:{s:o, s:o, s:o, s:o}, s:[]},"
     // Metadata
-    "s:{},"
+    "s:o,"
     // Filters {FilterList []}
     "s:{s:[]}"
     "}",
@@ -578,6 +537,7 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
             "QSV",
                 "Decode",       hb_value_bool(job->qsv.decode),
                 "AsyncDepth",   hb_value_int(job->qsv.async_depth),
+                "AdapterIndex", hb_value_int(adapter_index),
         "Audio",
             "CopyMask",
             "FallbackEncoder",  hb_value_int(job->acodec_fallback),
@@ -589,13 +549,13 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
                 "Default",      hb_value_bool(job->select_subtitle_config.default_track),
                 "Burn",         hb_value_bool(subtitle_search_burn),
             "SubtitleList",
-        "Metadata",
+        "Metadata",             hb_value_dup(job->metadata->dict),
         "Filters",
             "FilterList"
     );
     if (dict == NULL)
     {
-        hb_error("json pack failure: %s", error.text);
+        hb_error("hb_job_to_dict, json pack failure: %s", error.text);
         return NULL;
     }
     hb_dict_t *dest_dict = hb_dict_get(dict, "Destination");
@@ -624,29 +584,29 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     else if (job->pts_to_start != 0 || job->pts_to_stop != 0)
     {
         range_dict = hb_dict_init();
-        hb_dict_set(source_dict, "Type", hb_value_string("time"));
+        hb_dict_set(range_dict, "Type", hb_value_string("time"));
         if (job->pts_to_start > 0)
         {
-            hb_dict_set(source_dict, "Start", hb_value_int(job->pts_to_start));
+            hb_dict_set(range_dict, "Start", hb_value_int(job->pts_to_start));
         }
         if (job->pts_to_stop > 0)
         {
-            hb_dict_set(source_dict, "End",
+            hb_dict_set(range_dict, "End",
                         hb_value_int(job->pts_to_start + job->pts_to_stop));
         }
     }
     else if (job->frame_to_start != 0 || job->frame_to_stop != 0)
     {
         range_dict = hb_dict_init();
-        hb_dict_set(source_dict, "Type", hb_value_string("frame"));
+        hb_dict_set(range_dict, "Type", hb_value_string("frame"));
         if (job->frame_to_start > 0)
         {
-            hb_dict_set(source_dict, "Start",
+            hb_dict_set(range_dict, "Start",
                         hb_value_int(job->frame_to_start + 1));
         }
         if (job->frame_to_stop > 0)
         {
-            hb_dict_set(source_dict, "End",
+            hb_dict_set(range_dict, "End",
                         hb_value_int(job->frame_to_start + job->frame_to_stop));
         }
     }
@@ -660,8 +620,10 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
     hb_dict_set(source_dict, "Range", range_dict);
 
     hb_dict_t *video_dict = hb_dict_get(dict, "Video");
-    hb_dict_set(video_dict, "ColorFormat",
-                hb_value_int(job->pix_fmt));
+    hb_dict_set(video_dict, "ColorInputFormat",
+                hb_value_int(job->input_pix_fmt));
+    hb_dict_set(video_dict, "ColorOutputFormat",
+                hb_value_int(job->output_pix_fmt));
     hb_dict_set(video_dict, "ColorRange",
                 hb_value_int(job->color_range));
     hb_dict_set(video_dict, "ColorPrimaries",
@@ -686,6 +648,55 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
                     hb_value_int(job->color_matrix_override));
     }
 
+    // Mastering metadata
+    hb_dict_t *mastering_dict;
+    if (job->mastering.has_primaries || job->mastering.has_luminance)
+    {
+        mastering_dict = json_pack_ex(&error, 0,
+        "{"
+        // DisplayPrimaries[3][2]
+        "s:[[[ii],[ii]],[[ii],[ii]],[[ii],[ii]]],"
+        // WhitePoint[2],
+        "s:[[i,i],[i,i]],"
+        // MinLuminance, MaxLuminance, HasPrimaries, HasLuminance
+        "s:[i,i],s:[i,i],s:b,s:b"
+        "}",
+            "DisplayPrimaries", job->mastering.display_primaries[0][0].num,
+                                job->mastering.display_primaries[0][0].den,
+                                job->mastering.display_primaries[0][1].num,
+                                job->mastering.display_primaries[0][1].den,
+                                job->mastering.display_primaries[1][0].num,
+                                job->mastering.display_primaries[1][0].den,
+                                job->mastering.display_primaries[1][1].num,
+                                job->mastering.display_primaries[1][1].den,
+                                job->mastering.display_primaries[2][0].num,
+                                job->mastering.display_primaries[2][0].den,
+                                job->mastering.display_primaries[2][1].num,
+                                job->mastering.display_primaries[2][1].den,
+            "WhitePoint", job->mastering.white_point[0].num,
+                          job->mastering.white_point[0].den,
+                          job->mastering.white_point[1].num,
+                          job->mastering.white_point[1].den,
+            "MinLuminance", job->mastering.min_luminance.num,
+                            job->mastering.min_luminance.den,
+            "MaxLuminance", job->mastering.max_luminance.num,
+                            job->mastering.max_luminance.den,
+            "HasPrimaries", job->mastering.has_primaries,
+            "HasLuminance", job->mastering.has_luminance
+        );
+        hb_dict_set(video_dict, "Mastering", mastering_dict);
+    }
+
+    // Content Light Level metadata
+    hb_dict_t *coll_dict;
+    if (job->coll.max_cll && job->coll.max_fall)
+    {
+        coll_dict = json_pack_ex(&error, 0, "{s:i, s:i}",
+            "MaxCLL",  job->coll.max_cll,
+            "MaxFALL", job->coll.max_fall);
+        hb_dict_set(video_dict, "ContentLightLevel", coll_dict);
+    }
+
     if (job->vquality > HB_INVALID_VIDEO_QUALITY)
     {
         hb_dict_set(video_dict, "Quality", hb_value_double(job->vquality));
@@ -720,54 +731,6 @@ hb_dict_t* hb_job_to_dict( const hb_job_t * job )
         hb_dict_set(video_dict, "Options",
                     hb_value_string(job->encoder_options));
     }
-    hb_dict_t *meta_dict = hb_dict_get(dict, "Metadata");
-    if (job->metadata->name != NULL)
-    {
-        hb_dict_set(meta_dict, "Name", hb_value_string(job->metadata->name));
-    }
-    if (job->metadata->artist != NULL)
-    {
-        hb_dict_set(meta_dict, "Artist",
-                    hb_value_string(job->metadata->artist));
-    }
-    if (job->metadata->composer != NULL)
-    {
-        hb_dict_set(meta_dict, "Composer",
-                    hb_value_string(job->metadata->composer));
-    }
-    if (job->metadata->comment != NULL)
-    {
-        hb_dict_set(meta_dict, "Comment",
-                    hb_value_string(job->metadata->comment));
-    }
-    if (job->metadata->genre != NULL)
-    {
-        hb_dict_set(meta_dict, "Genre", hb_value_string(job->metadata->genre));
-    }
-    if (job->metadata->album != NULL)
-    {
-        hb_dict_set(meta_dict, "Album", hb_value_string(job->metadata->album));
-    }
-    if (job->metadata->album_artist != NULL)
-    {
-        hb_dict_set(meta_dict, "AlbumArtist",
-                    hb_value_string(job->metadata->album_artist));
-    }
-    if (job->metadata->description != NULL)
-    {
-        hb_dict_set(meta_dict, "Description",
-                    hb_value_string(job->metadata->description));
-    }
-    if (job->metadata->long_description != NULL)
-    {
-        hb_dict_set(meta_dict, "LongDescription",
-                    hb_value_string(job->metadata->long_description));
-    }
-    if (job->metadata->release_date != NULL)
-    {
-        hb_dict_set(meta_dict, "ReleaseDate",
-                    hb_value_string(job->metadata->release_date));
-    }
 
     // process chapter list
     hb_dict_t *chapter_list = hb_dict_get(dest_dict, "ChapterList");
@@ -926,6 +889,7 @@ typedef const char * const_str_t;
 
 static double*      unpack_f(double *f)     { return f; }
 static int*         unpack_i(int *i)        { return i; }
+static unsigned*    unpack_u(unsigned *u)   { return u; }
 static json_int_t*  unpack_I(json_int_t *i) { return i; }
 static int *        unpack_b(int *b)        { return b; }
 static const_str_t* unpack_s(const_str_t *s){ return s; }
@@ -1024,6 +988,8 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     hb_value_array_t * subtitle_list = NULL;
     hb_value_array_t * filter_list = NULL;
     hb_value_t       * mux = NULL, * vcodec = NULL;
+    hb_dict_t        * mastering_dict = NULL;
+    hb_dict_t        * coll_dict = NULL;
     hb_value_t       * acodec_copy_mask = NULL, * acodec_fallback = NULL;
     const char       * destfile = NULL;
     const char       * range_type = NULL;
@@ -1031,14 +997,11 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     const char       * video_profile = NULL, * video_level = NULL;
     const char       * video_options = NULL;
     int                subtitle_search_burn = 0;
-    const char       * meta_name = NULL, * meta_artist = NULL;
-    const char       * meta_album_artist = NULL, * meta_release = NULL;
-    const char       * meta_comment = NULL, * meta_genre = NULL;
-    const char       * meta_composer = NULL, * meta_desc = NULL;
-    const char       * meta_long_desc = NULL;
     json_int_t         range_start = -1, range_end = -1, range_seek_points = -1;
     int                vbitrate = -1;
     double             vquality = HB_INVALID_VIDEO_QUALITY;
+    int                adapter_index = -1;
+    hb_dict_t        * meta_dict = NULL;
 
     result = json_unpack_ex(dict, &error, 0,
     "{"
@@ -1054,23 +1017,26 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     "s?{s:i, s:i},"
     // Video {Codec, Quality, Bitrate, Preset, Tune, Profile, Level, Options
     //       TwoPass, Turbo,
-    //       ColorFormat, ColorRange,
+    //       ColorInputFormat, ColorOutputFormat, ColorRange,
     //       ColorPrimaries, ColorTransfer, ColorMatrix,
+    //       Mastering,
+    //       ContentLightLevel,
     //       ColorPrimariesOverride, ColorTransferOverride, ColorMatrixOverride,
-    //       QSV {Decode, AsyncDepth}}
-    "s:{s:o, s?f, s?i, s?s, s?s, s?s, s?s, s?s,"
+    //       QSV {Decode, AsyncDepth, AdapterIndex}}
+    "s:{s:o, s?F, s?i, s?s, s?s, s?s, s?s, s?s,"
     "   s?b, s?b,"
-    "   s?i, s?i,"
     "   s?i, s?i, s?i,"
     "   s?i, s?i, s?i,"
-    "   s?{s?b, s?i}},"
+    "   s?o,"
+    "   s?o,"
+    "   s?i, s?i, s?i,"
+    "   s?{s?b, s?i, s?i}},"
     // Audio {CopyMask, FallbackEncoder, AudioList}
     "s?{s?o, s?o, s?o},"
     // Subtitle {Search {Enable, Forced, Default, Burn}, SubtitleList}
     "s?{s?{s:b, s?b, s?b, s?b}, s?o},"
-    // Metadata {Name, Artist, Composer, AlbumArtist, ReleaseDate,
-    //           Comment, Genre, Description, LongDescription}
-    "s?{s?s, s?s, s?s, s?s, s?s, s?s, s?s, s?s, s?s},"
+    // Metadata
+    "s?o,"
     // Filters {FilterList}
     "s?{s?o}"
     "}",
@@ -1106,17 +1072,21 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
             "Options",              unpack_s(&video_options),
             "TwoPass",              unpack_b(&job->twopass),
             "Turbo",                unpack_b(&job->fastfirstpass),
-            "ColorFormat",          unpack_i(&job->pix_fmt),
+            "ColorInputFormat",     unpack_i(&job->input_pix_fmt),
+            "ColorOutputFormat",    unpack_i(&job->output_pix_fmt),
             "ColorRange",           unpack_i(&job->color_range),
             "ColorPrimaries",       unpack_i(&job->color_prim),
             "ColorTransfer",        unpack_i(&job->color_transfer),
             "ColorMatrix",          unpack_i(&job->color_matrix),
+            "Mastering",            unpack_o(&mastering_dict),
+            "ContentLightLevel",    unpack_o(&coll_dict),
             "ColorPrimariesOverride", unpack_i(&job->color_prim_override),
             "ColorTransferOverride",  unpack_i(&job->color_transfer_override),
             "ColorMatrixOverride",    unpack_i(&job->color_matrix_override),
             "QSV",
                 "Decode",           unpack_b(&job->qsv.decode),
                 "AsyncDepth",       unpack_i(&job->qsv.async_depth),
+                "AdapterIndex",     unpack_i(&adapter_index),
         "Audio",
             "CopyMask",             unpack_o(&acodec_copy_mask),
             "FallbackEncoder",      unpack_o(&acodec_fallback),
@@ -1128,16 +1098,7 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
                 "Default",          unpack_b(&job->select_subtitle_config.default_track),
                 "Burn",             unpack_b(&subtitle_search_burn),
             "SubtitleList",         unpack_o(&subtitle_list),
-        "Metadata",
-            "Name",                 unpack_s(&meta_name),
-            "Artist",               unpack_s(&meta_artist),
-            "Composer",             unpack_s(&meta_composer),
-            "AlbumArtist",          unpack_s(&meta_album_artist),
-            "ReleaseDate",          unpack_s(&meta_release),
-            "Comment",              unpack_s(&meta_comment),
-            "Genre",                unpack_s(&meta_genre),
-            "Description",          unpack_s(&meta_desc),
-            "LongDescription",      unpack_s(&meta_long_desc),
+        "Metadata",                 unpack_o(&meta_dict),
         "Filters",
             "FilterList",           unpack_o(&filter_list)
     );
@@ -1146,10 +1107,10 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
         hb_error("hb_dict_to_job: failed to parse dict: %s", error.text);
         goto fail;
     }
-
-    // Make sure QSV Decode is only True if the hardware is available.
-    job->qsv.decode = job->qsv.decode && hb_qsv_available();
-
+    if (meta_dict != NULL)
+    {
+        job->metadata->dict = hb_value_dup(meta_dict);
+    }
     // Lookup mux id
     if (hb_value_type(mux) == HB_VALUE_TYPE_STRING)
     {
@@ -1219,8 +1180,13 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
     hb_job_set_encoder_level(job, video_level);
     hb_job_set_encoder_options(job, video_options);
 
+#if HB_PROJECT_FEATURE_QSV
+    if (job->qsv.ctx) {
+        job->qsv.ctx->dx_index = adapter_index;
+    }
+#endif
     // If both vbitrate and vquality were specified, vbitrate is used;
-    // we need to ensure the unused rate contro mode is always set to an
+    // we need to ensure the unused rate control mode is always set to an
     // invalid value, as if both values are valid, behavior is undefined
     // (some encoders first check for a valid vquality, whereas others
     //  check for a valid vbitrate instead)
@@ -1238,103 +1204,60 @@ hb_job_t* hb_dict_to_job( hb_handle_t * h, hb_dict_t *dict )
 
     job->select_subtitle_config.dest = subtitle_search_burn ?
                                             RENDERSUB : PASSTHRUSUB;
-    if (meta_name != NULL)
+
+    if (mastering_dict != NULL)
     {
-        if (meta_name[0] != 0)
+        result = json_unpack_ex(mastering_dict, &error, 0,
+        "{"
+        // DisplayPrimaries[3][2]
+        "s:[[[ii],[ii]],[[ii],[ii]],[[ii],[ii]]],"
+        // WhitePoint[2],
+        "s:[[i,i],[i,i]],"
+        // MinLuminance, MaxLuminance, HasPrimaries, HasLuminance
+        "s:[i,i],s:[i,i],s:b,s:b"
+        "}",
+            "DisplayPrimaries", unpack_i(&job->mastering.display_primaries[0][0].num),
+                                unpack_i(&job->mastering.display_primaries[0][0].den),
+                                unpack_i(&job->mastering.display_primaries[0][1].num),
+                                unpack_i(&job->mastering.display_primaries[0][1].den),
+                                unpack_i(&job->mastering.display_primaries[1][0].num),
+                                unpack_i(&job->mastering.display_primaries[1][0].den),
+                                unpack_i(&job->mastering.display_primaries[1][1].num),
+                                unpack_i(&job->mastering.display_primaries[1][1].den),
+                                unpack_i(&job->mastering.display_primaries[2][0].num),
+                                unpack_i(&job->mastering.display_primaries[2][0].den),
+                                unpack_i(&job->mastering.display_primaries[2][1].num),
+                                unpack_i(&job->mastering.display_primaries[2][1].den),
+            "WhitePoint", unpack_i(&job->mastering.white_point[0].num),
+                          unpack_i(&job->mastering.white_point[0].den),
+                          unpack_i(&job->mastering.white_point[1].num),
+                          unpack_i(&job->mastering.white_point[1].den),
+            "MinLuminance", unpack_i(&job->mastering.min_luminance.num),
+                            unpack_i(&job->mastering.min_luminance.den),
+            "MaxLuminance", unpack_i(&job->mastering.max_luminance.num),
+                            unpack_i(&job->mastering.max_luminance.den),
+            "HasPrimaries", unpack_b(&job->mastering.has_primaries),
+            "HasLuminance", unpack_b(&job->mastering.has_luminance)
+        );
+        if (result < 0)
         {
-            hb_metadata_set_name(job->metadata, meta_name);
-        }
-        else
-        {
-            hb_metadata_set_name(job->metadata, NULL);
+            hb_error("hb_dict_to_job: failed to parse mastering_dict: %s", error.text);
+            goto fail;
         }
     }
-    if (meta_artist != NULL)
+
+    if (coll_dict != NULL)
     {
-        if (meta_artist[0] != 0)
+        result = json_unpack_ex(coll_dict, &error, 0,
+        // {MaxCLL, MaxFALL}
+        "{s:i, s:i}",
+            "MaxCLL",   unpack_u(&job->coll.max_cll),
+            "MaxFALL",  unpack_u(&job->coll.max_fall)
+        );
+        if (result < 0)
         {
-            hb_metadata_set_artist(job->metadata, meta_artist);
-        }
-        else
-        {
-            hb_metadata_set_artist(job->metadata, NULL);
-        }
-    }
-    if (meta_composer != NULL)
-    {
-        if (meta_composer[0] != 0)
-        {
-            hb_metadata_set_composer(job->metadata, meta_composer);
-        }
-        else
-        {
-            hb_metadata_set_composer(job->metadata, NULL);
-        }
-    }
-    if (meta_album_artist != NULL)
-    {
-        if (meta_album_artist[0] != 0)
-        {
-            hb_metadata_set_album_artist(job->metadata, meta_album_artist);
-        }
-        else
-        {
-            hb_metadata_set_album_artist(job->metadata, NULL);
-        }
-    }
-    if (meta_release != NULL)
-    {
-        if (meta_release[0] != 0)
-        {
-            hb_metadata_set_release_date(job->metadata, meta_release);
-        }
-        else
-        {
-            hb_metadata_set_release_date(job->metadata, NULL);
-        }
-    }
-    if (meta_comment != NULL)
-    {
-        if (meta_comment[0] != 0)
-        {
-            hb_metadata_set_comment(job->metadata, meta_comment);
-        }
-        else
-        {
-            hb_metadata_set_comment(job->metadata, NULL);
-        }
-    }
-    if (meta_genre != NULL)
-    {
-        if (meta_genre[0] != 0)
-        {
-            hb_metadata_set_genre(job->metadata, meta_genre);
-        }
-        else
-        {
-            hb_metadata_set_genre(job->metadata, NULL);
-        }
-    }
-    if (meta_desc != NULL)
-    {
-        if (meta_desc[0] != 0)
-        {
-            hb_metadata_set_description(job->metadata, meta_desc);
-        }
-        else
-        {
-            hb_metadata_set_description(job->metadata, NULL);
-        }
-    }
-    if (meta_long_desc != NULL)
-    {
-        if (meta_long_desc[0] != 0)
-        {
-            hb_metadata_set_long_description(job->metadata, meta_long_desc);
-        }
-        else
-        {
-            hb_metadata_set_long_description(job->metadata, NULL);
+            hb_error("hb_dict_to_job: failed to parse coll_dict: %s", error.text);
+            goto fail;
         }
     }
 
@@ -1765,10 +1688,13 @@ char* hb_set_anamorphic_size_json(const char * json_param)
     "s:{"
     //   Geometry {Width, Height, PAR {Num, Den}},
     "s:{s:i, s:i, s:{s:i, s:i}},"
-    //   AnamorphicMode, Keep, ItuPAR, Modulus, MaxWidth, MaxHeight,
-    "s:i, s?i, s?b, s:i, s:i, s:i,"
+    //   AnamorphicMode, Flags, Keep, ItuPAR, Modulus, MaxWidth, MaxHeight,
+    //   DisplayWidth, DisplayHeight
+    "s:i, s?i, s?i, s?b, s?i, s?i, s?i, s?i, s?i"
     //   Crop [Top, Bottom, Left, Right]
     "s?[iiii]"
+    //   Pad [Top, Bottom, Left, Right]
+    "s?[iiii]"
     "  }"
     "}",
     "SourceGeometry",
@@ -1785,15 +1711,22 @@ char* hb_set_anamorphic_size_json(const char * json_param)
                 "Num",          unpack_i(&ui_geo.geometry.par.num),
                 "Den",          unpack_i(&ui_geo.geometry.par.den),
         "AnamorphicMode",       unpack_i(&ui_geo.mode),
+        "Flags",                unpack_i(&ui_geo.flags),
         "Keep",                 unpack_i(&ui_geo.keep),
         "ItuPAR",               unpack_b(&ui_geo.itu_par),
         "Modulus",              unpack_i(&ui_geo.modulus),
         "MaxWidth",             unpack_i(&ui_geo.maxWidth),
         "MaxHeight",            unpack_i(&ui_geo.maxHeight),
+        "DisplayWidth",         unpack_i(&ui_geo.displayWidth),
+        "DisplayHeight",        unpack_i(&ui_geo.displayHeight),
         "Crop",                 unpack_i(&ui_geo.crop[0]),
                                 unpack_i(&ui_geo.crop[1]),
                                 unpack_i(&ui_geo.crop[2]),
-                                unpack_i(&ui_geo.crop[3])
+                                unpack_i(&ui_geo.crop[3]),
+        "Pad",                  unpack_i(&ui_geo.pad[0]),
+                                unpack_i(&ui_geo.pad[1]),
+                                unpack_i(&ui_geo.pad[2]),
+                                unpack_i(&ui_geo.pad[3])
     );
     hb_value_free(&dict);
 
@@ -1935,6 +1868,18 @@ char* hb_get_preview_json(hb_handle_t * h, const char *json_param)
     return result;
 }
 
+hb_image_t * hb_get_preview3_json(hb_handle_t * h, int picture, const char *json_job)
+{
+    hb_image_t * image;
+    hb_dict_t  * job_dict;
+
+    job_dict = hb_value_json(json_job);
+    image = hb_get_preview3(h, picture, job_dict);
+    hb_value_free(&job_dict);
+
+    return image;
+}
+
 char* hb_get_preview_params_json(int title_idx, int preview_idx,
                             int deinterlace, hb_geometry_settings_t *settings)
 {
