@@ -1,6 +1,6 @@
 /* ports.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -25,9 +25,13 @@
 #include <kernel/OS.h>
 #endif
 
-#if defined(SYS_DARWIN) || defined(SYS_FREEBSD)
+#if defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_NETBSD)
 #include <sys/types.h>
 #include <sys/sysctl.h>
+#if HB_PROJECT_FEATURE_QSV && defined(SYS_FREEBSD)
+#include <libdrm/drm.h>
+#include <fcntl.h>
+#endif
 #endif
 
 #ifdef SYS_OPENBSD
@@ -218,7 +222,7 @@ void hb_snooze( int delay )
     }
 #if defined( SYS_BEOS )
     snooze( 1000 * delay );
-#elif defined( SYS_DARWIN ) || defined( SYS_LINUX ) || defined( SYS_FREEBSD) || defined( SYS_SunOS )
+#elif defined( SYS_DARWIN ) || defined( SYS_LINUX ) || defined( SYS_FREEBSD) || defined(SYS_NETBSD) || defined( SYS_SunOS )
     usleep( 1000 * delay );
 #elif defined( SYS_CYGWIN ) || defined( SYS_MINGW )
     Sleep( delay );
@@ -279,8 +283,12 @@ const char* hb_get_cpu_platform_name()
             return "Intel microarchitecture Airmont";
         case HB_CPU_PLATFORM_INTEL_KBL:
             return "Intel microarchitecture Kaby Lake";
+        case HB_CPU_PLATFORM_INTEL_CML:
+            return "Intel microarchitecture Comet Lake";
         case HB_CPU_PLATFORM_INTEL_ICL:
             return "Intel microarchitecture Ice Lake";
+        case HB_CPU_PLATFORM_INTEL_TGL:
+            return "Intel microarchitecture Tiger Lake";
         default:
             return NULL;
     }
@@ -371,8 +379,18 @@ static void init_cpu_info()
                     case 0x9E:
                         hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_KBL;
                         break;
+                    case 0xA5:
+                    case 0xA6:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_CML;
+                        break;
+                    case 0x7D:
                     case 0x7E:
                         hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_ICL;
+                        break;
+                    case 0x8C:
+                    case 0x8D:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_TGL;
+                        break;
                     default:
                         break;
                 }
@@ -443,9 +461,9 @@ static int init_cpu_count()
     get_system_info( &info );
     cpu_count = info.cpu_count;
 
-#elif defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_OPENBSD)
+#elif defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_NETBSD) || defined(SYS_OPENBSD)
     size_t length = sizeof( cpu_count );
-#ifdef SYS_OPENBSD
+#if defined SYS_OPENBSD || defined(SYS_NETBSD)
     int mib[2] = { CTL_HW, HW_NCPU };
     if( sysctl(mib, 2, &cpu_count, &length, NULL, 0) )
 #else
@@ -629,7 +647,7 @@ char * hb_get_temporary_directory()
     else
         base = strdup("/tmp");
 #endif
-    /* I prefer to remove evntual last '/' (for cygwin) */
+    /* I prefer to remove eventual last '/' (for cygwin) */
     if( base[strlen(base)-1] == '/' )
         base[strlen(base)-1] = '\0';
 
@@ -640,7 +658,7 @@ char * hb_get_temporary_directory()
 }
 
 /************************************************************************
- * Get a tempory filename for HB
+ * Get a temporary filename for HB
  ***********************************************************************/
 char * hb_get_temporary_filename( char *fmt, ... )
 {
@@ -848,7 +866,7 @@ static uint64_t hb_thread_to_integer( const hb_thread_t* t )
  * hb_thread_func()
  ************************************************************************
  * We use it as the root routine for any thread, for two reasons:
- *  + To set the thread priority on OS X (pthread_setschedparam() could
+ *  + To set the thread priority on macOS (pthread_setschedparam() could
  *    be called from hb_thread_init(), but it's nicer to do it as we
  *    are sure it is done before the real routine starts)
  *  + Get informed when the thread exits, so we know whether
@@ -858,8 +876,9 @@ static void attribute_align_thread hb_thread_func( void * _t )
 {
     hb_thread_t * t = (hb_thread_t *) _t;
 
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
-    /* Set the thread priority */
+#if (defined( SYS_DARWIN ) && !defined(__aarch64__)) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined(SYS_NETBSD)
+    /* Set the thread priority
+       Do not change priority on Darwin arm systems */
     struct sched_param param;
     memset( &param, 0, sizeof( struct sched_param ) );
     param.sched_priority = t->priority;
@@ -1005,7 +1024,7 @@ hb_lock_t * hb_lock_init()
 
     pthread_mutexattr_init(&mta);
 
-#if defined( SYS_CYGWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
+#if defined( SYS_CYGWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined(SYS_NETBSD)
     pthread_mutexattr_settype(&mta, PTHREAD_MUTEX_NORMAL);
 #endif
 
@@ -1518,7 +1537,7 @@ char * hb_strndup(const char * src, size_t len)
 }
 
 #if HB_PROJECT_FEATURE_QSV
-#ifdef SYS_LINUX
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD)
 
 #define MAX_NODES             16
 #define DRI_RENDER_NODE_START 128
@@ -1676,7 +1695,7 @@ void hb_display_close(hb_display_t ** _d)
     *_d = NULL;
 }
 
-#else // !SYS_LINUX
+#else // !SYS_LINUX && !SYS_FREEBSD
 
 hb_display_t * hb_display_init(const char         *  driver_name,
                                const char * const * interface_names)
@@ -1689,7 +1708,7 @@ void hb_display_close(hb_display_t ** _d)
     (void)_d;
 }
 
-#endif // SYS_LINUX
+#endif // SYS_LINUX || SYS_FREEBSD
 #else // !HB_PROJECT_FEATURE_QSV
 
 hb_display_t * hb_display_init(const char         *  driver_name,
