@@ -1,6 +1,6 @@
 /* scan.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -844,7 +844,7 @@ static int DecodePreviews( hb_scan_t * data, hb_title_t * title, int flush )
 
         if( data->store_previews )
         {
-            hb_save_preview( data->h, title->index, i, vid_buf );
+            hb_save_preview( data->h, title->index, i, vid_buf, HB_PREVIEW_FORMAT_JPG );
         }
 
         /* Detect black borders */
@@ -1084,12 +1084,33 @@ skip_preview:
         }
 
         hb_log( "scan: %d previews, %dx%d, %.3f fps, autocrop = %d/%d/%d/%d, "
-                "aspect %s, PAR %d:%d",
+                "aspect %s, PAR %d:%d, color profile: %d-%d-%d",
                 npreviews, title->geometry.width, title->geometry.height,
                 (float)title->vrate.num / title->vrate.den,
                 title->crop[0], title->crop[1], title->crop[2], title->crop[3],
                 aspect_to_string(&title->dar),
-                title->geometry.par.num, title->geometry.par.den);
+                title->geometry.par.num, title->geometry.par.den,
+                title->color_prim, title->color_transfer, title->color_matrix);
+
+        if (title->mastering.has_primaries || title->mastering.has_luminance)
+        {
+            hb_log("scan: mastering display metadata: r(%5.4f,%5.4f) g(%5.4f,%5.4f) b(%5.4f %5.4f) wp(%5.4f, %5.4f) min_luminance=%f, max_luminance=%f",
+                   hb_q2d(title->mastering.display_primaries[0][0]),
+                   hb_q2d(title->mastering.display_primaries[0][1]),
+                   hb_q2d(title->mastering.display_primaries[1][0]),
+                   hb_q2d(title->mastering.display_primaries[1][1]),
+                   hb_q2d(title->mastering.display_primaries[2][0]),
+                   hb_q2d(title->mastering.display_primaries[2][1]),
+                   hb_q2d(title->mastering.white_point[0]), hb_q2d(title->mastering.white_point[1]),
+                   hb_q2d(title->mastering.min_luminance), hb_q2d(title->mastering.max_luminance));
+        }
+
+        if (title->coll.max_cll || title->coll.max_fall)
+        {
+            hb_log("scan: content light level: max_cll=%u, max_fall=%u",
+                   title->coll.max_cll,
+                   title->coll.max_fall);
+        }
 
         if (title->video_decode_support != HB_DECODE_SUPPORT_SW)
         {
@@ -1302,6 +1323,9 @@ static void LookForAudio(hb_scan_t *scan, hb_title_t * title, hb_buffer_t * b)
                 case HB_ACODEC_FFFLAC:
                     codec_name = "FLAC";
                     break;
+                case HB_ACODEC_MP2:
+                    codec_name = "MPEG";
+                    break;
                 case HB_ACODEC_MP3:
                     codec_name = "MP3";
                     break;
