@@ -1,6 +1,6 @@
 /* work.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2021 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -42,7 +42,7 @@ static void filter_loop( void * );
 /**
  * Allocates work object and launches work thread with work_func.
  * @param jobs Handle to hb_list_t.
- * @param die Handle to user inititated exit indicator.
+ * @param die Handle to user initiated exit indicator.
  * @param error Handle to error indicator.
  */
 hb_thread_t * hb_work_init( hb_list_t * jobs, volatile int * die, hb_error_code * error, hb_job_t ** job )
@@ -141,6 +141,12 @@ static void work_func( void * _work )
             hb_job_close(&job);
             job = new_job;
         }
+#if HB_PROJECT_FEATURE_QSV
+        if (hb_qsv_available())
+        {
+            hb_qsv_setup_job(job);
+        }
+#endif
         hb_job_setup_passes(job->h, job, passes);
         hb_job_close(&job);
 
@@ -293,6 +299,17 @@ hb_work_object_t* hb_video_encoder(hb_handle_t *h, int vcodec)
             w->codec_param = AV_CODEC_ID_H264;
             break;
         case HB_VCODEC_FFMPEG_VT_H265:
+        case HB_VCODEC_FFMPEG_VT_H265_10BIT:
+            w = hb_get_work(h, WORK_ENCAVCODEC);
+            w->codec_param = AV_CODEC_ID_HEVC;
+            break;
+#endif
+#if HB_PROJECT_FEATURE_MF
+        case HB_VCODEC_FFMPEG_MF_H264:
+            w = hb_get_work(h, WORK_ENCAVCODEC);
+            w->codec_param = AV_CODEC_ID_H264;
+            break;
+        case HB_VCODEC_FFMPEG_MF_H265:
             w = hb_get_work(h, WORK_ENCAVCODEC);
             w->codec_param = AV_CODEC_ID_HEVC;
             break;
@@ -427,13 +444,13 @@ void hb_display_job_info(hb_job_t *job)
 #if HB_PROJECT_FEATURE_QSV
     if (hb_qsv_decode_is_enabled(job))
     {
-        hb_log("   + decoder: %s",
-               hb_qsv_decode_get_codec_name(title->video_codec_param));
+        hb_log("   + decoder: %s %d-bit (%s)",
+               hb_qsv_decode_get_codec_name(title->video_codec_param), hb_get_bit_depth(job->input_pix_fmt), av_get_pix_fmt_name(job->input_pix_fmt));
     }
     else
 #endif
     {
-        hb_log("   + decoder: %s", title->video_codec_name);
+        hb_log("   + decoder: %s %d-bit (%s)", title->video_codec_name, hb_get_bit_depth(job->input_pix_fmt), av_get_pix_fmt_name(job->input_pix_fmt));
     }
 
     if( title->video_bitrate )
@@ -539,6 +556,9 @@ void hb_display_job_info(hb_job_t *job)
                 case HB_VCODEC_FFMPEG_NVENC_H265:
                 case HB_VCODEC_FFMPEG_VT_H264:
                 case HB_VCODEC_FFMPEG_VT_H265:
+                case HB_VCODEC_FFMPEG_VT_H265_10BIT:
+                case HB_VCODEC_FFMPEG_MF_H264:
+                case HB_VCODEC_FFMPEG_MF_H265:
                     hb_log("     + profile: %s", job->encoder_profile);
                 default:
                     break;
@@ -563,6 +583,9 @@ void hb_display_job_info(hb_job_t *job)
                 case HB_VCODEC_FFMPEG_VT_H264:
                 // VT h.265 currently only supports auto level
                 // case HB_VCODEC_FFMPEG_VT_H265:
+                // MF h.264/h.265 currently only supports auto level
+                // case HB_VCODEC_FFMPEG_MF_H264:
+                // case HB_VCODEC_FFMPEG_MF_H265:
                     hb_log("     + level:   %s", job->encoder_level);
                 default:
                     break;
@@ -593,6 +616,28 @@ void hb_display_job_info(hb_job_t *job)
 
         hb_log("     + color profile: %d-%d-%d",
                job->color_prim, job->color_transfer, job->color_matrix);
+
+        if (job->color_transfer == HB_COLR_TRA_SMPTEST2084)
+        {
+            if (job->mastering.has_primaries || job->mastering.has_luminance)
+            {
+                hb_log("     + mastering display metadata: r(%5.4f,%5.4f) g(%5.4f,%5.4f) b(%5.4f %5.4f) wp(%5.4f, %5.4f) min_luminance=%f, max_luminance=%f",
+                       hb_q2d(job->mastering.display_primaries[0][0]),
+                       hb_q2d(job->mastering.display_primaries[0][1]),
+                       hb_q2d(job->mastering.display_primaries[1][0]),
+                       hb_q2d(job->mastering.display_primaries[1][1]),
+                       hb_q2d(job->mastering.display_primaries[2][0]),
+                       hb_q2d(job->mastering.display_primaries[2][1]),
+                       hb_q2d(job->mastering.white_point[0]), hb_q2d(job->mastering.white_point[1]),
+                       hb_q2d(job->mastering.min_luminance), hb_q2d(job->mastering.max_luminance));
+            }
+            if (job->coll.max_cll && job->coll.max_fall)
+            {
+                hb_log("     + content light level: max_cll=%u, max_fall=%u",
+                       job->coll.max_cll,
+                       job->coll.max_fall);
+            }
+        }
     }
 
     if (job->indepth_scan)
@@ -703,7 +748,8 @@ void hb_display_job_info(hb_job_t *job)
                 {
                     hb_log( "   + dynamic range compression: %f", audio->config.out.dynamic_range_compression );
                 }
-                if (hb_audio_dither_is_supported(audio->config.out.codec))
+                if (hb_audio_dither_is_supported(audio->config.out.codec,
+                                            audio->config.in.sample_bit_depth))
                 {
                     hb_log("   + dither: %s",
                            hb_audio_dither_get_description(audio->config.out.dither_method));
@@ -1203,7 +1249,8 @@ static int sanitize_audio(hb_job_t *job)
         }
 
         /* sense-check the requested dither */
-        if (hb_audio_dither_is_supported(audio->config.out.codec))
+        if (hb_audio_dither_is_supported(audio->config.out.codec,
+                                         audio->config.in.sample_bit_depth))
         {
             if (audio->config.out.dither_method ==
                 hb_audio_dither_get_default())
@@ -1224,179 +1271,10 @@ static int sanitize_audio(hb_job_t *job)
     return 0;
 }
 
-static int sanitize_qsv( hb_job_t * job )
+static void sanitize_filter_list(hb_job_t *job, hb_geometry_t src_geo)
 {
-#if HB_PROJECT_FEATURE_QSV
-#if 0 // TODO: re-implement QSV VPP filtering and QSV zerocopy path
-    int i;
+    hb_list_t *list = job->list_filter;
 
-    /*
-     * XXX: mfxCoreInterface's CopyFrame doesn't work in old drivers, and our
-     *      workaround is really slow. If we have validated CPU-based filters in
-     *      the list and we can't use CopyFrame, disable QSV decoding until a
-     *      better solution is implemented.
-     */
-    if (hb_qsv_copyframe_is_slow(job->vcodec))
-    {
-        if (job->list_filter != NULL)
-        {
-            int encode_only = 0;
-            for (i = 0; i < hb_list_count(job->list_filter) && !encode_only; i++)
-            {
-                hb_filter_object_t *filter = hb_list_item(job->list_filter, i);
-                switch (filter->id)
-                {
-                    // validated, CPU-based filters
-                    case HB_FILTER_ROTATE:
-                    case HB_FILTER_RENDER_SUB:
-                    case HB_FILTER_AVFILTER:
-                        encode_only = 1;
-                        break;
-
-                    // CPU-based deinterlace (validated)
-                    case HB_FILTER_DEINTERLACE:
-                    {
-                        int mode = hb_dict_get_int(filter->settings, "mode");
-                        if (!(mode & MODE_DEINTERLACE_QSV))
-                        {
-                            encode_only = 1;
-                        }
-                    } break;
-
-                    // other filters will be removed
-                    default:
-                        break;
-                }
-            }
-            if (encode_only)
-            {
-                hb_log("do_job: QSV: possible CopyFrame bug, using encode-only path");
-                if (hb_get_cpu_platform() >= HB_CPU_PLATFORM_INTEL_IVB)
-                {
-                    hb_log("do_job: QSV: please update your Intel graphics driver to version 9.18.10.3257 or later");
-                }
-                job->qsv.decode = 0;
-            }
-        }
-    }
-
-    /*
-     * When QSV's VPP is used for filtering, not all CPU filters
-     * are supported, so we need to do a little extra setup here.
-     */
-    if (job->vcodec & HB_VCODEC_QSV_MASK)
-    {
-        int vpp_settings[7];
-        int num_cpu_filters = 0;
-        hb_filter_object_t *filter;
-        // default values for VPP filter
-        vpp_settings[0] = job->title->geometry.width;
-        vpp_settings[1] = job->title->geometry.height;
-        vpp_settings[2] = job->title->crop[0];
-        vpp_settings[3] = job->title->crop[1];
-        vpp_settings[4] = job->title->crop[2];
-        vpp_settings[5] = job->title->crop[3];
-        vpp_settings[6] = 0; // deinterlace: off
-        if (job->list_filter != NULL && hb_list_count(job->list_filter) > 0)
-        {
-            while (hb_list_count(job->list_filter) > num_cpu_filters)
-            {
-                filter = hb_list_item(job->list_filter, num_cpu_filters);
-                switch (filter->id)
-                {
-                    // cropping and scaling always done via VPP filter
-                    case HB_FILTER_CROP_SCALE:
-                        hb_dict_extract_int(&vpp_settings[0], filter->settings,
-                                            "width");
-                        hb_dict_extract_int(&vpp_settings[1], filter->settings,
-                                            "height");
-                        hb_dict_extract_int(&vpp_settings[2], filter->settings,
-                                            "crop-top");
-                        hb_dict_extract_int(&vpp_settings[3], filter->settings,
-                                            "crop-bottom");
-                        hb_dict_extract_int(&vpp_settings[4], filter->settings,
-                                            "crop-left");
-                        hb_dict_extract_int(&vpp_settings[5], filter->settings,
-                                            "crop-right");
-
-                        hb_list_rem(job->list_filter, filter);
-                        hb_filter_close(&filter);
-                        break;
-
-                    // pick VPP or CPU deinterlace depending on settings
-                    case HB_FILTER_DEINTERLACE:
-                    {
-                        int mode = hb_dict_get_int(filter->settings, "mode");
-                        if (mode & MODE_DEINTERLACE_QSV)
-                        {
-                            // deinterlacing via VPP filter
-                            vpp_settings[6] = 1;
-                            hb_list_rem(job->list_filter, filter);
-                            hb_filter_close(&filter);
-                        }
-                        else
-                        {
-                            // validated
-                            num_cpu_filters++;
-                        }
-                    } break;
-
-                    // then, validated filters
-                    case HB_FILTER_ROTATE: // TODO: use Media SDK for this
-                    case HB_FILTER_RENDER_SUB:
-                    case HB_FILTER_AVFILTER:
-                        num_cpu_filters++;
-                        break;
-
-                    // finally, drop all unsupported filters
-                    default:
-                        hb_log("do_job: QSV: full path, removing unsupported filter '%s'",
-                               filter->name);
-                        hb_list_rem(job->list_filter, filter);
-                        hb_filter_close(&filter);
-                        break;
-                }
-            }
-            if (num_cpu_filters > 0)
-            {
-                // we need filters to copy to system memory and back
-                filter = hb_filter_init(HB_FILTER_QSV_PRE);
-                hb_add_filter_dict(job, filter, NULL);
-                filter = hb_filter_init(HB_FILTER_QSV_POST);
-                hb_add_filter_dict(job, filter, NULL);
-            }
-            if (vpp_settings[0] != job->title->geometry.width  ||
-                vpp_settings[1] != job->title->geometry.height ||
-                vpp_settings[2] >= 1 /* crop */       ||
-                vpp_settings[3] >= 1 /* crop */       ||
-                vpp_settings[4] >= 1 /* crop */       ||
-                vpp_settings[5] >= 1 /* crop */       ||
-                vpp_settings[6] >= 1 /* deinterlace */)
-            {
-                // we need the VPP filter
-                hb_dict_t * dict = hb_dict_init();
-                hb_dict_set(dict, "width", hb_value_int(vpp_settings[0]));
-                hb_dict_set(dict, "height", hb_value_int(vpp_settings[1]));
-                hb_dict_set(dict, "crop-top", hb_value_int(vpp_settings[2]));
-                hb_dict_set(dict, "crop-bottom", hb_value_int(vpp_settings[3]));
-                hb_dict_set(dict, "crop-left", hb_value_int(vpp_settings[4]));
-                hb_dict_set(dict, "crop-right", hb_value_int(vpp_settings[5]));
-                hb_dict_set(dict, "deinterlace", hb_value_int(vpp_settings[6]));
-
-                filter = hb_filter_init(HB_FILTER_QSV);
-                hb_add_filter_dict(job, filter, dict);
-                hb_value_free(&dict);
-            }
-        }
-    }
-#endif // QSV VPP filtering and QSV zerocopy path
-#endif // HB_PROJECT_FEATURE_QSV
-
-    return 0;
-}
-
-static void sanitize_filter_list(hb_list_t *list, hb_geometry_t src_geo)
-{
     // Add selective deinterlacing mode if comb detection is enabled
     if (hb_filter_find(list, HB_FILTER_COMB_DETECT) != NULL)
     {
@@ -1459,6 +1337,54 @@ static void sanitize_filter_list(hb_list_t *list, hb_geometry_t src_geo)
             }
         }
     }
+
+#if HB_PROJECT_FEATURE_QSV && (defined( _WIN32 ) || defined( __MINGW32__ ))
+        // sanitize_qsv looks for subtitle render filter, so must happen after
+        // sanitize_subtitle
+        if (hb_qsv_is_enabled(job))
+        {
+            hb_qsv_sanitize_filter_list(job);
+            // In case of video memory need to set formats after satitize filters 
+            // to ensure that full QSV pipeline supported
+            if (hb_qsv_full_path_is_enabled(job))
+            {
+                // Formats supported in QSV pipeline via video memory
+                if (job->input_pix_fmt == AV_PIX_FMT_YUV420P10)
+                {
+                    job->input_pix_fmt = AV_PIX_FMT_P010LE;
+                }
+                else if (job->input_pix_fmt == AV_PIX_FMT_YUV420P)
+                {
+                    job->input_pix_fmt = AV_PIX_FMT_NV12;
+                }
+            }
+        }
+#endif
+
+    if (hb_video_encoder_pix_fmt_is_supported(job->vcodec, job->input_pix_fmt) == 0)
+    {
+        // Some encoders require a specific input pixel format
+        // that could be different from the current pipeline format.
+        const int *encoder_pix_fmts = hb_video_encoder_get_pix_fmts(job->vcodec);
+        int encoder_pix_fmt = *encoder_pix_fmts;
+
+#if HB_PROJECT_FEATURE_QSV && (defined( _WIN32 ) || defined( __MINGW32__ ))
+        if (!hb_qsv_full_path_is_enabled(job))
+        {
+            // Formats supported by QSV pipeline via system memory
+            if (encoder_pix_fmt != AV_PIX_FMT_YUV420P10 &&
+                encoder_pix_fmt != AV_PIX_FMT_YUV420P)
+            {
+                // TODO: skip the first AV_PIX_FMT_NV12 or AV_PIX_FMT_P010LE formats, prefer second format for system memory
+                encoder_pix_fmts++;
+                encoder_pix_fmt = *encoder_pix_fmts;
+            }
+        }
+#endif
+
+        hb_filter_object_t *filter = hb_filter_init(HB_FILTER_FORMAT);
+        hb_add_filter(job, filter, hb_strdup_printf("format=%s", av_get_pix_fmt_name(encoder_pix_fmt)));
+    }
 }
 
 /**
@@ -1518,42 +1444,29 @@ static void do_job(hb_job_t *job)
         *job->die = 1;
         goto cleanup;
     }
-
-    // sanitize_qsv looks for subtitle render filter, so must happen after
-    // sanitize_subtitle
-    result = sanitize_qsv(job);
-    if (result)
-    {
-        *job->done_error = HB_ERROR_WRONG_INPUT;
-        *job->die = 1;
-        goto cleanup;
-    }
     // Filters have an effect on settings.
     // So initialize the filters and update the job.
     if (job->list_filter && hb_list_count(job->list_filter))
     {
         hb_filter_init_t init;
 
-        sanitize_filter_list(job->list_filter, title->geometry);
+        // Select the optimal pixel format
+        // for the pipeline
+        job->input_pix_fmt = hb_get_best_pix_fmt(job);
+
+        sanitize_filter_list(job, title->geometry);
 
         memset(&init, 0, sizeof(init));
         init.time_base.num = 1;
         init.time_base.den = 90000;
         init.job = job;
-        // TODO: When more complete pix format support is complete this
-        // needs to be updated to reflect the pix_fmt output by
-        // decavcodec.c.  This may be different than title->pix_fmt
-        // since we will likely only support planar YUV color formats.
-        init.pix_fmt = AV_PIX_FMT_YUV420P;
+        init.pix_fmt = job->input_pix_fmt;
         init.color_range = AVCOL_RANGE_MPEG;
 
         init.color_prim = title->color_prim;
         init.color_transfer = title->color_transfer;
         init.color_matrix = title->color_matrix;
-        init.geometry.width = title->geometry.width;
-        init.geometry.height = title->geometry.height;
-
-        init.geometry.par = job->par;
+        init.geometry = title->geometry;
         memset(init.crop, 0, sizeof(int[4]));
         init.vrate = job->vrate;
         init.cfr = 0;
@@ -1573,9 +1486,17 @@ static void do_job(hb_job_t *job)
             }
             i++;
         }
+        job->output_pix_fmt = init.pix_fmt;
+        job->color_prim = init.color_prim;
+        job->color_transfer = init.color_transfer;
+        job->color_matrix = init.color_matrix;
+        job->color_range = init.color_range;
         job->width = init.geometry.width;
         job->height = init.geometry.height;
-        job->par = init.geometry.par;
+        // job->par is supplied by the frontend.
+        //
+        // The filter chain does not know what the final desired PAR is.
+        // job->par = init.geometry.par;
         memcpy(job->crop, init.crop, sizeof(int[4]));
         job->vrate = init.vrate;
         job->cfr = init.cfr;
@@ -1627,33 +1548,14 @@ static void do_job(hb_job_t *job)
     hb_reduce(&job->vrate.num, &job->vrate.den,
                job->vrate.num,  job->vrate.den);
 
-#if HB_PROJECT_FEATURE_QSV
-#if 0 // TODO: re-implement QSV zerocopy path
-    if (hb_qsv_decode_is_enabled(job) && (job->vcodec & HB_VCODEC_QSV_MASK))
+    job->fifo_mpeg2  = hb_fifo_init( FIFO_SMALL, FIFO_SMALL_WAKE );
+    job->fifo_raw    = hb_fifo_init( FIFO_SMALL, FIFO_SMALL_WAKE );
+    if (!job->indepth_scan)
     {
-        job->fifo_mpeg2  = hb_fifo_init( FIFO_MINI, FIFO_MINI_WAKE );
-        job->fifo_raw    = hb_fifo_init( FIFO_MINI, FIFO_MINI_WAKE );
-        if (!job->indepth_scan)
-        {
-            // When doing subtitle indepth scan, the pipeline ends at sync
-            job->fifo_sync   = hb_fifo_init( FIFO_MINI, FIFO_MINI_WAKE );
-            job->fifo_render = hb_fifo_init( FIFO_MINI, FIFO_MINI_WAKE );
-            job->fifo_mpeg4  = hb_fifo_init( FIFO_MINI, FIFO_MINI_WAKE );
-        }
-    }
-    else
-#endif // QSV zerocopy path
-#endif
-    {
-        job->fifo_mpeg2  = hb_fifo_init( FIFO_SMALL, FIFO_SMALL_WAKE );
-        job->fifo_raw    = hb_fifo_init( FIFO_SMALL, FIFO_SMALL_WAKE );
-        if (!job->indepth_scan)
-        {
-            // When doing subtitle indepth scan, the pipeline ends at sync
-            job->fifo_sync   = hb_fifo_init( FIFO_SMALL, FIFO_SMALL_WAKE );
-            job->fifo_render = NULL; // Attached to filter chain
-            job->fifo_mpeg4  = hb_fifo_init( FIFO_LARGE, FIFO_LARGE_WAKE );
-        }
+        // When doing subtitle indepth scan, the pipeline ends at sync
+        job->fifo_sync   = hb_fifo_init( FIFO_SMALL, FIFO_SMALL_WAKE );
+        job->fifo_render = NULL; // Attached to filter chain
+        job->fifo_mpeg4  = hb_fifo_init( FIFO_LARGE, FIFO_LARGE_WAKE );
     }
 
     result = sanitize_audio(job);
@@ -1980,8 +1882,13 @@ cleanup:
     {
         analyze_subtitle_scan(job);
     }
-
     hb_buffer_pool_free();
+#if HB_PROJECT_FEATURE_QSV
+    if (!job->indepth_scan && hb_qsv_is_enabled(job))
+    {
+        hb_qsv_context_uninit(job);
+    }
+#endif
 }
 
 static inline void copy_chapter( hb_buffer_t * dst, hb_buffer_t * src )
@@ -2086,7 +1993,7 @@ void hb_work_loop( void * _w )
  * Loops calling work function for associated filter object.
  * Sleeps when fifo is full.
  * Monitors work done indicator.
- * Exits loop when work indiactor is set.
+ * Exits loop when work indicator is set.
  * @param _w Handle to work object.
  */
 static void filter_loop( void * _f )
